import numpy as np
import scipy
import matplotlib.pyplot as plt
import scipy.stats.distributions as dists
from scipy import stats
from copy import deepcopy
import matplotlib.gridspec as gridspec
from sklearn.metrics import auc

import h5py
import matplotlib
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42


data_file = './data/WeidemannKahana2016_data.hdf5'
data = h5py.File(data_file, 'r')['WeidemannKahana2016_data']


# Create filter for invalid confidence responses
conf_filt = (np.isfinite(data['recog_conf']) &
             (data['recog_conf'] <= 5) &
             (data['recog_conf'] >= 1))

# Create filter for invalid RTs:
rt_filt = (np.isfinite(data['recog_rt']) &
           (data['recog_rt'] <= 3000) &
           (data['recog_rt'] >= 300))

# remove invalid rts and confidence ratings
data = data[conf_filt & rt_filt]

# len(conf_filt)-len(data) -> 24647
# len(conf_filt) -> 907909
# len(data) -> 883262
# 24647./907909 -> 0.027146993806648024


# Filter out all sessions where there isn't at least 1 trial for each
# combination of target:"old" / "new" lure:"old" / "new"
oldnewtarglure_filt = np.ones(len(data), np.bool)
targfilt = data['type'] == 'RECOG_TARGET'
lurefilt = data['type'] == 'RECOG_LURE'

for subj in np.unique(data['participant']):
    subjfilt = data['participant'] == subj
    for sess in np.unique(data[subjfilt]['session']):
        subjsessfilt = subjfilt & (data['session'] == sess)
        targsubsessfilt = subjsessfilt & targfilt
        luresubsessfilt = subjsessfilt & lurefilt
        if((np.var(data[targsubsessfilt]['recog_resp'] == 'old') == 0.) |
           (np.var(data[luresubsessfilt]['recog_resp'] == 'old') == 0.)):
            oldnewtarglure_filt[subjsessfilt] = False

data_targlurefilt = data[oldnewtarglure_filt]
#  (883262-853624) / 883262. = 0.033555162567845104

exclsessions_targlure = []
for subj in np.unique(data[~oldnewtarglure_filt]['participant']):
    for sess in np.unique(data[
            (~oldnewtarglure_filt) &
            (data['participant'] == subj)]['session']):
        exclsessions_targlure.append((subj, sess))
# 121 sessions    


# Filter out all sessions where there isn't at least 1 trial for each
# combination of target: "old" / "new" / recalled / unrecalled
# lure: "old" / "new":
oldnewtarglure_filt_recunrec = np.ones(len(data), np.bool)
oldnewtarglure_thresh_recunrec = 1
recalledfilt = data['recalled'] == 'yes'
unrecalledfilt = data['recalled'] == 'no'
oldfilt = data['recog_resp'] == 'old'
for subj in np.unique(data['participant']):
    subjfilt = data['participant'] == subj
    for sess in np.unique(data[subjfilt]['session']):
        subjsessfilt = subjfilt & (data['session'] == sess)
        targsubsessfilt_recalled = subjsessfilt & recalledfilt & targfilt
        targsubsessfilt_unrecalled = subjsessfilt & unrecalledfilt & targfilt
        luresubsessfilt = subjsessfilt & lurefilt
        targsubsessfilt_recalled_old = targsubsessfilt_recalled & oldfilt
        targsubsessfilt_unrecalled_old = targsubsessfilt_unrecalled & oldfilt
        luresubsessfilt_old = luresubsessfilt & oldfilt
        if((np.sum(targsubsessfilt_recalled_old) <
            oldnewtarglure_thresh_recunrec) |
           ((targsubsessfilt_recalled.sum() -
             np.sum(targsubsessfilt_recalled_old)) <
            oldnewtarglure_thresh_recunrec) |
           (np.sum(targsubsessfilt_unrecalled_old) <
            oldnewtarglure_thresh_recunrec) |
           ((targsubsessfilt_unrecalled.sum() -
            np.sum(targsubsessfilt_unrecalled_old)) <
            oldnewtarglure_thresh_recunrec) |
           (np.sum(luresubsessfilt_old) <
            oldnewtarglure_thresh_recunrec) |
           ((luresubsessfilt.sum() -
             np.sum(luresubsessfilt_old)) < oldnewtarglure_thresh_recunrec)):
            oldnewtarglure_filt_recunrec[
                subjsessfilt] = False

data_recunrecfilt = data[oldnewtarglure_filt_recunrec]

exclsessions_recunrec = []
for subj in np.unique(data[~oldnewtarglure_filt_recunrec]['participant']):
    for sess in np.unique(data[
            (~oldnewtarglure_filt_recunrec) &
            (data['participant'] == subj)]['session']):
        exclsessions_recunrec.append((subj, sess))
# 661 sessions (- 121 sessions = 540 additionally excluded sessions)


def get_hitfa(evs):
    # Compute hits and FAs for each subj/session:
    hitfa = np.recarray(
        len(evs), dtype=[('hits', np.float), ('fas', np.float)])
    for dv in hitfa.dtype.names:
        hitfa[dv] *= np.nan
    targfilt = evs['type'] == 'RECOG_TARGET'
    lurefilt = evs['type'] == 'RECOG_LURE'
    oldfilt = evs['recog_resp'] == 'old'
    for subj in np.unique(evs['participant']):
        subjfilt = evs['participant'] == subj
        for sess in np.unique(
                evs[subjfilt]['session']):
            subjsessfilt = subjfilt & (evs['session'] == sess)
            hits = np.sum(subjsessfilt & targfilt & oldfilt) / np.float(
                np.sum(subjsessfilt & targfilt))
            fas = np.sum(subjsessfilt & lurefilt & oldfilt) / np.float(
                np.sum(subjsessfilt & lurefilt))
            hitfa['hits'][subjsessfilt] = hits
            hitfa['fas'][subjsessfilt] = fas
    return(hitfa)

hitfa_recunrec = get_hitfa(data_recunrecfilt)
hitfa_targlure = get_hitfa(data_targlurefilt)


def get_sesscounts(evs, printsess=False):
    sesscounts = {}
    for subj in np.unique(evs['participant']):
        try:
            sesscounts[np.unique(
                evs[evs['participant'] == subj][
                    'session']).shape[0]] += 1
        except KeyError:
            sesscounts[np.unique(
                evs[evs['participant'] == subj][
                    'session']).shape[0]] = 1
        if printsess:
            print subj, np.unique(evs[evs['participant'] == subj][
                'session']).shape[0], np.unique(evs[
                    evs['participant'] == subj]['session'])
    return(sesscounts)

sesscount_recunrec = get_sesscounts(data_recunrecfilt)

total_sessions_recunrec = 0
for sessions_recunrec in sesscount_recunrec:
    total_sessions_recunrec += sessions_recunrec * sesscount_recunrec[
        sessions_recunrec]
# 2580 sessions

sesscount_targlure = get_sesscounts(data_targlurefilt)

total_sessions_targlure = 0
for sessions_targlure in sesscount_targlure:
    total_sessions_targlure += sessions_targlure*sesscount_targlure[
        sessions_targlure]
# 3120 sessions


sesscount_orig = get_sesscounts(data, printsess=False)

total_sessions_orig = 0
for sessions_orig in sesscount_orig:
    total_sessions_orig += sessions_orig*sesscount_orig[sessions_orig]
# 3241 sessions


def get_resprates_conf(
        events, confs, type_field='type', targ_val='RECOG_TARGET',
        lure_val='RECOG_LURE', recog_conf_field='recog_conf',
        recog_resp_field='recog_resp', old_val='old', new_val='new'):
    """
    Get the response rates for target (hit/miss) and lure (FA/CR)
    trials and corresponding response and trial counts based on
    confidence bins.

    Parameters
    ----------
    events : array
        All events to be analyzed. Must have dtype with fields (as in
        recarray)
    confs : {array_like}
        Container of all possible confidence values.
    type_field : string,optional
        Field in events array specifying whether a given trial is a
        target or lure trial.
    targ_val : {any dtype},optional
        Value specifying that a trial is a target trial.
    lure_val : {any dtype},optional
        Value specifying that a trial is a lure trial.
    recog_conf_field: str,optional
        Field in events array specifying the confidence response in a
        given recognition trial.
    recog_resp_field : str,optional
        Field in events array specifying the recognition
        response ("old" or "new") in a given trial.
    old_val : {any dtype},optional
        Value specifying that an "old" response was made in a given trial
    new_vale : {any dtype},optional
        Value specifying that a "new" response was made in a given trial.

    Returns
    -------
    resp_rates : dictionary
        Dictionary with "targetpres" ("hits") and "lurepres" ("FAs")
        proportions and coresponding response counts
        ("targpres_respcounts" and "lurepres_respcounts" respectively).
    """
    trgs = events[events[type_field] == targ_val]
    lrs = events[events[type_field] == lure_val]
    targpres_respcounts = []
    lurepres_respcounts = []
    for cnf in reversed(confs):
        targpres_respcounts.append(
            np.sum(trgs[trgs[recog_conf_field] == cnf][
                recog_resp_field] == new_val))
        lurepres_respcounts.append(
            np.sum(lrs[lrs[recog_conf_field] == cnf][
                recog_resp_field] == new_val))
    for cnf in confs:
        targpres_respcounts.append(
            np.sum(
                trgs[trgs[recog_conf_field] == cnf][
                    recog_resp_field] == old_val))
        lurepres_respcounts.append(
            np.sum(lrs[lrs[recog_conf_field] == cnf][
                recog_resp_field] == old_val))
    resp_rates = {}
    resp_rates['targpres'] = (np.array(
        targpres_respcounts, dtype=np.float128) /
                            np.sum(targpres_respcounts))
    resp_rates['lurepres'] = (np.array(
        lurepres_respcounts, dtype=np.float128) /
                              np.sum(lurepres_respcounts))
    resp_rates['targpres_respcounts'] = np.array(
        targpres_respcounts, dtype=np.float128)
    resp_rates['lurepres_respcounts'] = np.array(
        lurepres_respcounts, dtype=np.float128)
    return resp_rates


def get_resprates_rt(
        events, rt_bins, rt_min=None, rt_max=None, type_field='type',
        targ_val='RECOG_TARGET', lure_val='RECOG_LURE',
        recog_rt_field='recog_rt', recog_resp_field='recog_resp',
        old_val='old', new_val='new'):
    """
    Get the response rates for target (hit/miss) and lure (FA/CR)
    trials and corresponding response and trial counts based on RT bins.

    Parameters
    ----------
    events : array
        All events to be analyzed. Must have dtype with fields (as in
        recarray)
    rt_bins : dictionary
        Dictionary with the response classes (e.g., 'old' or 'new') as
        keys and raw counts or proportions of observations that should
        be in each bin (e.g., to set the proportion in each bin equal
        to those for confidence ratings).
    rt_min : {int,float},optional
        Minimal RT to include in analyses.
    rt_max : {int,float},optional
        Maximal RT to include in analyses.
    type_field : string,optional
        Field in events array specifying whether a given trial is a
        target or lure trial.
    targ_val : {any dtype},optional
        Value specifying that a trial is a target trial.
    lure_val : {any dtype},optional
        Value specifying that a trial is a lure trial.
    recog_rt_field: str,optional
        Field in events array specifying the latency for a given
        recognition response.
    recog_resp_field : str,optional
        Field in events array specifying the recognition
        response ("old" or "new") in a given trial.
    old_val : {any dtype},optional
        Value specifying that an "old" response was made in a given trial
    new_vale : {any dtype},optional
        Value specifying that a "new" response was made in a given trial.

    Returns
    -------
    resp_rates : dictionary
        Dictionary with "targetpres" ("hits" and technically misses)
        and "lurepres" ("FAs" and technically CRs) proportions,
        coresponding response counts ("targpres_respcounts" and
        "lurepres_respcounts" respectively), targpres_respprops
        (proportion of respones when the target was present), and a
        set of indices for ordering the above in terms of accuracy
        (within each classification response).
    classpoint_indx : {int, np.nan}
        Index of classification point.
    """
    # Filter for the valid RTs:
    if rt_min:
        rt_min_filt = np.float32(events[recog_rt_field]) >= rt_min
    else:
        rt_min_filt = np.ones(len(events[recog_rt_field]), np.bool)
    if rt_max:
        rt_max_filt = np.float32(events[recog_rt_field]) <= rt_max
    else:
        rt_max_filt = np.ones(len(events[recog_rt_field]), np.bool)
    rt_indx = ((np.isfinite(np.float32(events[recog_rt_field]))) &
               rt_min_filt & rt_max_filt)
    # Filter for old/new responses:
    old_indx = events[recog_resp_field] == old_val
    new_indx = events[recog_resp_field] == new_val

    # get sorted old/new RTs:
    recog_rts = {}
    recog_rts['old'] = np.sort(events[rt_indx & old_indx][recog_rt_field])
    recog_rts['new'] = np.sort(events[rt_indx & new_indx][recog_rt_field])

    # if there is no more than one response class, hits & FAs are undefined:
    num_bins = 0
    for key in rt_bins:
        num_bins += len(rt_bins[key])
    if (len(recog_rts['old']) == 0) or (len(recog_rts['new']) == 0):
        resp_rates = {'targpres': np.zeros(num_bins) * np.nan,
                      'lurepres': np.zeros(num_bins) * np.nan,
                      'targpres_respcounts': np.zeros(num_bins) * np.nan,
                      'lurepres_respcounts': np.zeros(num_bins) * np.nan,
                      'targpres_respprops': np.zeros(num_bins) * np.nan,
                      'ordered_indices': np.zeros(num_bins) * np.nan}
        return resp_rates, np.nan
    rt_binprops = {}
    rt_cutoffs = {}
    for key in rt_bins:
        rt_binprops[key] = (np.array(rt_bins[key], dtype=np.float128) /
                            np.array(rt_bins[key]).sum())
        rt_binprops[key] = np.r_[0, np.int32(np.round(
            np.cumsum(rt_binprops[key]) * (len(recog_rts[key]) - 1)))]
        rt_cutoffs[key] = [recog_rts[key][i] for i in rt_binprops[key]]

    # separate target and lure events:
    targs = events[events[type_field] == targ_val]
    targsRTs = np.float32(targs[recog_rt_field])
    targsRTs_finite = np.isfinite(targsRTs)
    lures = events[events[type_field] == lure_val]
    luresRTs = np.float32(lures[recog_rt_field])
    luresRTs_finite = np.isfinite(luresRTs)

    # counts for responses when the target was present (hit/miss) or
    # when the lure was present (FA/CR):
    targpres_respcounts = []
    lurepres_respcounts = []
    for i in range(0, len(rt_cutoffs['new']) - 1):
        # Filters for valid target and lure RTs within response bin:
        rtindx_trg = (targsRTs_finite & (targsRTs > rt_cutoffs['new'][i]) &
                      (targsRTs <= rt_cutoffs['new'][i+1]))
        rtindx_lrs = (luresRTs_finite & (luresRTs > rt_cutoffs['new'][i]) &
                      (luresRTs <= rt_cutoffs['new'][i+1]))
        # miss counts (misses will be plotted on the 'hits' axis for
        # ROC):
        targpres_respcounts.append(np.sum(
            targs[rtindx_trg][recog_resp_field] == new_val))
        # correct rejection counts (CRs will be plotted on the 'FA'
        # axis for ROC):
        lurepres_respcounts.append(np.sum(
            lures[rtindx_lrs][recog_resp_field] == new_val))
    class_point_index = i
    for i in range(len(rt_cutoffs['old'])-1, 0, -1):
        # Filters for valid target and lure RTs within response bin:
        rtindx_trg = (targsRTs_finite & (targsRTs <= rt_cutoffs['old'][i]) &
                      (targsRTs > rt_cutoffs['old'][i-1]))
        rtindx_lrs = (luresRTs_finite & (luresRTs <= rt_cutoffs['old'][i]) &
                      (luresRTs > rt_cutoffs['old'][i-1]))
        # hit counts:
        targpres_respcounts.append(np.sum(
            targs[rtindx_trg][recog_resp_field] == old_val))
        # FA counts:
        lurepres_respcounts.append(np.sum(
            lures[rtindx_lrs][recog_resp_field] == old_val))

    # add missing first values
    rtindx_trg = targsRTs == rt_cutoffs['new'][0]
    rtindx_lrs = luresRTs == rt_cutoffs['new'][0]
    targpres_respcounts[0] += np.sum(
        targs[rtindx_trg][recog_resp_field] == new_val)
    lurepres_respcounts[0] += np.sum(
        lures[rtindx_lrs][recog_resp_field] == new_val)

    rtindx_trg = targsRTs == rt_cutoffs['old'][0]
    rtindx_lrs = luresRTs == rt_cutoffs['old'][0]
    targpres_respcounts[-1] += np.sum(
        targs[rtindx_trg][recog_resp_field] == old_val)
    lurepres_respcounts[-1] += np.sum(
        lures[rtindx_lrs][recog_resp_field] == old_val)

    # calculate response rates and return:
    resp_rates = {}
    resp_rates['targpres'] = (np.array(
        targpres_respcounts, dtype=np.float128) /
                              np.sum(targpres_respcounts))
    resp_rates['lurepres'] = (np.array(
        lurepres_respcounts, dtype=np.float128) /
                              np.sum(lurepres_respcounts))
    resp_rates['targpres_respcounts'] = np.array(targpres_respcounts,
                                                 dtype=np.float128)
    resp_rates['lurepres_respcounts'] = np.array(lurepres_respcounts,
                                                 dtype=np.float128)
    # proportion of respones when the target was present
    resp_rates['targpres_respprops'] = (resp_rates['targpres_respcounts'] /
                                        (resp_rates['targpres_respcounts'] +
                                         resp_rates['lurepres_respcounts']))
    resp_rates['ordered_indices'] = np.r_[
        np.argsort(resp_rates['targpres_respprops'][:class_point_index+1]),
        np.argsort(resp_rates['targpres_respprops'][class_point_index+1:]) +
        class_point_index + 1]
    return resp_rates, class_point_index


def get_POCs_byvars_confrt(
        events, confs, rt_bins=None, variables=[], rt_min=None,
        rt_max=None, type_field='type', targ_val='RECOG_TARGET',
        lure_val='RECOG_LURE', recog_conf_field='recog_conf',
        recog_rt_field='recog_rt', recog_resp_field='recog_resp',
        old_val='old', new_val='new', ordered=False, correct_zROCs=False):
    """
    Aggregate ROCs by variables (e.g., subjects, sessions). If variables
    is an empty list data is treated like coming from one
    subject/session.
    """
    import itertools
    if not rt_bins:
        rt_bins = {'old': [1] * len(confs),
                   'new': [1] * len(confs)}
    # get resprates for empty recarray just to get the right dictionaries:
    print('>>> BEGINNING: Two intended (but irrelevant) warnings:')
    resprates_conf = get_resprates_conf(
        events[np.zeros(len(events), dtype=bool)], confs)
    resprates_rt, cpi = get_resprates_rt(
        events[np.zeros(len(events), dtype=bool)], rt_bins)
    print('<<< END: Two intended (but irrelevant) warnings.')
    # replace values in dictionaries (and cpi) by empty lists to be
    # filled in as specified in function call:
    for key in resprates_conf:
        resprates_conf[key] = []
    for key in resprates_rt:
        resprates_rt[key] = []
    for key in variables:
        resprates_conf[key] = []
        resprates_rt[key] = []
    cpi = []

    filters = []
    for var in variables:
        levels = np.unique(events[var])
        filters.append([events[var] == levels[i]
                        for i in range(len(levels))])    
    for selection in itertools.product(*filters):
        datslice = np.sum(selection, 0) == len(selection)
        if np.sum(datslice) == 0:
            continue
        selevs = events[datslice]
        selresprates_conf = get_resprates_conf(selevs, confs)
        selresprates_rt, selcpi = get_resprates_rt(selevs, rt_bins)
        for key in selresprates_conf:
            resprates_conf[key].append(selresprates_conf[key])
        for key in selresprates_rt:
            resprates_rt[key].append(selresprates_rt[key])
        for key in variables:
            if len(np.unique(selevs[key])) != 1:
                raise ValueError('Non-unique var: ' +
                                 str(np.unique(selevs[key])))
            resprates_conf[key].append(selevs[key][0])
            resprates_rt[key].append(selevs[key][0])
        cpi.append(selcpi)

    resprates_dict = {
        'conf': resprates_conf.copy(),
        'rt': resprates_rt.copy()}
    resprates_n = len(resprates_dict['conf']['targpres'])

    resprates_dict_agg = {
        'all': {'mean': {'conf': {}, 'rt': {}},
                'se': {'conf': {}, 'rt': {}},
                'ci95': {'conf': {}, 'rt': {}},
                'corr': {'conf': None, 'rt': None},
                'n': {'conf': resprates_n, 'rt': resprates_n}},
        'indiv_resprates': {'conf': None, 'rt': None},
        'cpi': np.nan}
    for var in variables:
        resprates_dict_agg[var] = {}
        for level in np.unique(events[var]):
            resprates_dict_agg[var][level] = {}
            for measure in resprates_dict_agg['all']:
                resprates_dict_agg[var][level][measure] = {}
                for dv in resprates_dict_agg['all'][measure]:
                    resprates_dict_agg[var][level][measure][dv] = {}

    for dv in resprates_dict.keys():
        for key in resprates_dict[dv].keys():
            if (key == 'ordered_indices') | (key in variables):
                continue
            for measure in resprates_dict_agg['all']:
                if measure in ['corr', 'n']:
                    continue
                resprates_dict_agg['all'][measure][dv][key] = None
            for var in resprates_dict_agg:
                if var in ['all', 'cpi', 'indiv_resprates']:
                    continue
                for level in resprates_dict_agg[var]:
                    for measure in resprates_dict_agg[var][level]:
                        if measure in ['corr', 'n']:
                            continue
                        resprates_dict_agg[var][level][measure][dv][key] = None
            resprates_dict[dv][key] = np.array(resprates_dict[dv][key])
            if ordered & ('ordered_indices' in resprates_dict[dv].keys()):
                for i in range(len(resprates_dict[dv][key])):
                    resprates_dict[dv][key][i] = resprates_dict[dv][key][i][
                        resprates_dict[dv]['ordered_indices'][i]]
        # create roc & poc data:
        resprates_dict[dv]['roc'] = {}
        resprates_dict[dv]['zroc'] = {}

        for measure in resprates_dict_agg['all']:
            if measure in ['corr', 'n']:
                continue
            resprates_dict_agg['all'][measure][dv]['roc'] = {}
            resprates_dict_agg['all'][measure][dv]['zroc'] = {}
            resprates_dict_agg['all'][measure][dv]['poc'] = {}
            resprates_dict_agg['all'][measure][dv]['zpoc'] = {}

        for var in resprates_dict_agg:
            if var in ['all', 'cpi', 'indiv_resprates']:
                continue
            for level in resprates_dict_agg[var]:
                for measure in resprates_dict_agg[var][level]:
                    if measure in ['corr', 'n']:
                        continue
                    resprates_dict_agg[var][level][measure][dv]['roc'] = {}
                    resprates_dict_agg[var][level][measure][dv]['zroc'] = {}
                    resprates_dict_agg[var][level][measure][dv]['poc'] = {}
                    resprates_dict_agg[var][level][measure][dv]['zpoc'] = {}
        for key in ['targpres', 'lurepres']:
            resprates_dict[dv]['roc'][key] = 1 - resprates_dict[
                dv][key].cumsum(1)
            # correct rounding errors:
            resprates_dict[dv]['roc'][key][
                resprates_dict[dv]['roc'][key] > 1] = 1
            resprates_dict[dv]['roc'][key][
                resprates_dict[dv]['roc'][key] < 0] = 0

            resprates_dict[dv]['zroc'][key] = dists.norm.ppf(
                np.float64(resprates_dict[dv]['roc'][key]))
            if correct_zROCs:
                # correct invalid zroc data:
                resprates_dict[dv]['zroc'][key][
                    resprates_dict[dv]['zroc'][key] ==
                    np.inf] = dists.norm.ppf(
                        (resprates_n - 0.5) / resprates_n)
                resprates_dict[dv]['zroc'][key][
                    resprates_dict[dv]['zroc'][
                        key] == -np.inf] = dists.norm.ppf(
                        0.5 / resprates_n)
        # create poc data:
        resprates_dict[dv]['poc'] = resprates_dict[dv]['roc'].copy()
        resprates_dict[dv]['poc']['lurepres'] = 1 - resprates_dict[dv]['roc'][
            'lurepres']
        resprates_dict[dv]['zpoc'] = resprates_dict[dv]['zroc'].copy()
        resprates_dict[dv]['zpoc']['lurepres'] = dists.norm.ppf(
            1 - np.float64(resprates_dict[dv]['roc']['lurepres']))
        if correct_zROCs:
            # correct invalid zpoc data:
            resprates_dict[dv]['zpoc']['lurepres'][
                resprates_dict[dv]['zpoc']['lurepres'] ==
                np.inf] = dists.norm.ppf(
                    (resprates_n - 0.5) / resprates_n)
            resprates_dict[dv]['zpoc']['lurepres'][
                resprates_dict[dv]['zpoc'][
                    'lurepres'] == -np.inf] = dists.norm.ppf(
                    0.5 / resprates_n)
    for dv in resprates_dict.keys():
        resprates_dict_agg['all']['corr'][dv] = {'luretargprescorr': [
            np.corrcoef(resprates_dict[dv]['lurepres'][:, i],
                        resprates_dict[dv]['targpres'][:, i])[0, 1]
            for i in range(resprates_dict[dv]['lurepres'].shape[1])]}
        for var in variables:
            for level in np.unique(resprates_dict[dv][var]):
                selection = np.array(resprates_dict[dv][var]) == level
                resprates_dict_agg[var][level]['n'][dv] = np.sum(
                    selection)
        for key in resprates_dict[dv].keys():
            if (key == 'ordered_indices') | (key in variables):
                continue
            if isinstance(resprates_dict[dv][key], np.ndarray):
                resprates_dict_agg['all']['mean'][dv][key] = resprates_dict[
                    dv][key].mean(0)
                resprates_dict_agg['all']['se'][dv][key] = (
                    np.std(resprates_dict[dv][key], axis=0, ddof=1) /
                    np.sqrt(resprates_dict_agg['all']['n'][dv]))
                resprates_dict_agg['all']['ci95'][dv][key] = (
                    resprates_dict_agg['all']['se'][dv][key] * dists.t.ppf(
                        0.975, resprates_dict_agg['all']['n'][dv] - 1))
                for var in variables:
                    for level in np.unique(resprates_dict[dv][var]):
                        selection = np.array(resprates_dict[dv][var]) == level
                        resprates_sel = resprates_dict[dv][key][selection]
                        resprates_dict_agg[var][level]['mean'][dv][
                            key] = resprates_sel.mean(0)
                        resprates_dict_agg[var][level]['se'][dv][key] = (
                            np.std(resprates_sel, axis=0, ddof=1) /
                            np.sqrt(resprates_dict_agg[var][level]['n'][dv]))
                        resprates_dict_agg[var][level]['ci95'][dv][key] = (
                            resprates_dict_agg[var][level]['se'][dv][key] *
                            dists.t.ppf(0.975, resprates_dict_agg[var][level][
                                'n'][dv] - 1))
            else:
                for key2 in resprates_dict[dv][key]:
                    resprates_dict_agg['all']['mean'][dv][key][
                        key2] = resprates_dict[dv][key][key2].mean(0)
                    resprates_dict_agg['all']['se'][dv][key][key2] = (
                        np.std(resprates_dict[dv][key][key2], axis=0, ddof=1) /
                        np.sqrt(resprates_dict_agg['all']['n'][dv]))
                    resprates_dict_agg['all']['ci95'][dv][key][key2] = (
                        np.std(resprates_dict[dv][key][key2], axis=0, ddof=1) /
                        np.sqrt(resprates_dict_agg['all']['n'][dv]) *
                        dists.t.ppf(
                            0.975, resprates_dict_agg['all']['n'][dv] - 1))
                    for var in variables:
                        for level in np.unique(resprates_dict[dv][var]):
                            selection = np.array(
                                resprates_dict[dv][var]) == level
                            resprates_sel = resprates_dict[dv][key][key2][
                                selection]
                            resprates_dict_agg[var][level]['mean'][dv][
                                key][key2] = resprates_sel.mean(0)
                            resprates_dict_agg[var][level]['se'][dv][key][
                                key2] = (
                                    np.std(resprates_sel, axis=0, ddof=1) /
                                    np.sqrt(resprates_dict_agg[var][
                                        level]['n'][dv]))
                            resprates_dict_agg[var][level]['ci95'][dv][key][
                                key2] = (resprates_dict_agg[var][level]['se'][
                                    dv][key][key2] * dists.t.ppf(
                                        0.975, resprates_dict_agg[var][
                                            level]['n'][dv] - 1))
    if len(np.unique(cpi)) != 1:
        raise ValueError("Invalid classification point indices:\n" + str(cpi))
    else:
        cpi = cpi[0]
    resprates_dict_agg['cpi'] = cpi
    resprates_dict_agg['indiv_resprates'] = resprates_dict
    return(resprates_dict_agg)


def get_resprates(events):
    rt_bins = {'old': [1] * 500, 'new': [1] * 500}
    resprates_dict_agg = get_POCs_byvars_confrt(
        events, variables=['participant', 'session'],
        confs=np.arange(1, 6), ordered=False, rt_bins=rt_bins)
    rt_bins = {'old': [1] * 5, 'new': [1] * 5}
    resprates_dict_agg_10 = get_POCs_byvars_confrt(
        events, variables=['participant', 'session'],
        confs=np.arange(1, 6), ordered=False, rt_bins=rt_bins)
    return(resprates_dict_agg, resprates_dict_agg_10)


def get_distplots(resprates):
    resprates = {'all': deepcopy(resprates[0]['indiv_resprates']),
                 'bins': deepcopy(resprates[1]['indiv_resprates'])}
    filter_indx = None
    allbins = ['all', 'bins']
    dvs = ['conf', 'rt']
    distplots = {}
    for ab in allbins:
        distplots[ab] = {}
        for dv in dvs:
            distplots[ab][dv] = {}
            for tl in ['targ', 'lure']:
                distplots[ab][dv][tl] = {}
                for on in ['old', 'new']:
                    distplots[ab][dv][tl][on] = {'dat': None, 'ROC': None}
            for ss in ['subj', 'sess']:
                distplots[ab][dv][ss] = None
    for allbins in distplots:
        for dv in distplots[allbins]:
            for targlure in ['targ', 'lure']:
                distplots[allbins][dv]['subj'] = np.array(
                    resprates[allbins][dv]['participant'])
                distplots[allbins][dv]['sess'] = np.array(
                    resprates[allbins][dv]['session'])
                distplots[allbins][dv][targlure]['old']['dat'] = (
                    resprates[allbins][dv][targlure+'pres'][
                        :, np.int(resprates[allbins][dv][
                            targlure+'pres'].shape[1]/2.):])
                distplots[allbins][dv][targlure]['new']['dat'] = (
                    resprates[allbins][dv][targlure+'pres'][
                        :, np.int(resprates[allbins][dv][
                            targlure+'pres'].shape[1]/2.)-1::-1])
            # keep only those subjects/sessions that have both "old"
            # and "new" responses for both targets and lures:
            tmpfilt = np.c_[
                distplots[allbins][dv]['targ']['old']['dat'].sum(1),
                distplots[allbins][dv]['targ']['new']['dat'].sum(1),
                distplots[allbins][dv]['lure']['old']['dat'].sum(1),
                distplots[allbins][dv]['lure']['new']['dat'].sum(1)].min(1) > 0
            if filter_indx is None:
                filter_indx = tmpfilt
            elif np.any(filter_indx != tmpfilt):
                raise ValueError('Invalid filter: ' + str(filter_indx)+'\n\n' +
                                 str(tmpfilt))
            for subjsess in ['subj', 'sess']:
                distplots[allbins][dv][subjsess] = distplots[
                    allbins][dv][subjsess][filter_indx]
            for targlure in ['targ', 'lure']:
                distplots[allbins][dv][targlure]['old']['dat'] = distplots[
                    allbins][dv][targlure]['old']['dat'][filter_indx]
                distplots[allbins][dv][targlure]['new']['dat'] = distplots[
                    allbins][dv][targlure]['new']['dat'][filter_indx]
                for oldnew in distplots[allbins][dv][targlure]:
                    distplots[allbins][dv][targlure][oldnew][
                        'dat'] /= np.repeat(
                            [distplots[allbins][dv][targlure][oldnew][
                                'dat'].sum(1)],
                            distplots[allbins][dv][targlure][oldnew][
                                'dat'].shape[1],
                        0).T
                    distplots[allbins][dv][targlure][oldnew]['ROC'] = np.c_[
                        np.ones(distplots[
                            allbins][dv][targlure][oldnew]['dat'].shape[0]),
                        1-distplots[allbins][dv][
                            targlure][oldnew]['dat'].cumsum(1)]
                    distplots[allbins][dv][targlure][oldnew]['ROC'][
                        distplots[allbins][dv][targlure][oldnew][
                            'ROC'] > 1] = 1
                    distplots[allbins][dv][targlure][oldnew]['ROC'][
                        distplots[allbins][dv][targlure][oldnew][
                            'ROC'] < 0] = 0
    return(distplots, filter_indx)


def get_distplots_subjmean(distplots):
    """ Works for distplots and distplots_fullroc """
    distplots_subjmean = {}
    for datpartition in distplots:
        distplots_subjmean[datpartition] = {}
        for allbins in distplots[datpartition]:
            distplots_subjmean[datpartition][allbins] = {}
            for dv in distplots[datpartition][allbins]:
                distplots_subjmean[datpartition][allbins][dv] = {}
                num_subj = len(np.unique(distplots[
                    datpartition][allbins][dv]['subj']))
                distplots_subjmean[
                    datpartition][allbins][dv]['subj'] = np.empty(
                        num_subj, 'S6')
                distplots_subjmean[datpartition][allbins][dv][
                    'num_of_sessions'] = np.ones(num_subj, np.int) * -1
                for targlure in ['targ', 'lure']:
                    distplots_subjmean[datpartition][allbins][
                        dv][targlure] = {}
                    for oldnew in distplots[datpartition][allbins][
                                dv][targlure]:
                        distplots_subjmean[datpartition][allbins][
                            dv][targlure][oldnew] = {}
                        for datroc in distplots[datpartition][
                                allbins][dv][targlure][oldnew]:
                            array_shape = list(np.shape(distplots[
                                datpartition][allbins][dv][
                                    targlure][oldnew][datroc]))
                            array_shape[0] = num_subj
                            distplots_subjmean[datpartition][allbins][
                                dv][targlure][oldnew][datroc] = np.ones(
                                    array_shape, np.float128)*np.nan
                for s, subj in enumerate(np.unique(distplots[
                        datpartition][allbins][dv]['subj'])):
                    subjfilt = distplots[
                        datpartition][allbins][dv]['subj'] == subj
                    distplots_subjmean[
                        datpartition][allbins][dv]['subj'][s] = subj
                    distplots_subjmean[
                        datpartition][allbins][dv]['num_of_sessions'][s] = len(
                            np.unique(distplots[
                                datpartition][allbins][dv]['sess'][subjfilt]))
                    for targlure in ['targ', 'lure']:
                        for oldnew in distplots[datpartition][allbins][
                                dv][targlure]:
                            for datroc in distplots[datpartition][
                                    allbins][dv][targlure][oldnew]:
                                distplots_subjmean[datpartition][
                                    allbins][dv][targlure][oldnew][
                                        datroc][s] = distplots[
                                            datpartition][allbins][dv][
                                                targlure][oldnew][datroc][
                                                    subjfilt].mean(0)
    return(distplots_subjmean)


def plot_barpanel(ax, dat, oldnew, targcol='blue', lurecol='orange',
                  targhatch='/', lurehatch='\\', selector=None):
    if selector is None:
        targdat = dat['targ'][oldnew]['dat'].mean(0)
        luredat = dat['lure'][oldnew]['dat'].mean(0)
    else:
        targdat = dat['targ'][oldnew]['dat'][selector].mean(0)
        luredat = dat['lure'][oldnew]['dat'][selector].mean(0)
    ax.bar(np.arange(0.6, 5.6, 1), targdat, alpha=0.3,
           hatch=targhatch, ec='black', fc=targcol)
    ax.bar(np.arange(0.6, 5.6, 1), targdat,
           hatch=targhatch, ec='black', fc='none', linewidth=2.,
           linestyle='solid', label='target')
    ax.bar(np.arange(0.6, 5.6, 1), luredat, alpha=0.3,
           hatch=lurehatch, ec='black', fc=lurecol)
    ax.bar(np.arange(0.6, 5.6, 1), luredat,
           hatch=lurehatch, ec='black', fc='none', linewidth=2.,
           linestyle='dashed', label='lure')
    ax.set_ylim((0, 1))
    ax.set_xlim((0.4, 5.6))
    ax.set_xticks(np.arange(1, 6))


def plot_rocpanel(ax, dat, datbins=None, selector=None):
    if selector is None:
        luredat = {'old': dat['lure']['old']['ROC'].mean(0),
                   'new': dat['lure']['new']['ROC'].mean(0)}
        targdat = {'old': dat['targ']['old']['ROC'].mean(0),
                   'new': dat['targ']['new']['ROC'].mean(0)}
        if datbins is not None:
            luredatbins = {'old': datbins['lure']['old']['ROC'].mean(0),
                           'new': datbins['lure']['new']['ROC'].mean(0)}
            targdatbins = {'old': datbins['targ']['old']['ROC'].mean(0),
                           'new': datbins['targ']['new']['ROC'].mean(0)}
    else:
        luredat = {'old': dat['lure']['old']['ROC'][selector].mean(0),
                   'new': dat['lure']['new']['ROC'][selector].mean(0)}
        targdat = {'old': dat['targ']['old']['ROC'][selector].mean(0),
                   'new': dat['targ']['new']['ROC'][selector].mean(0)}
        if datbins is not None:
            luredatbins = {
                'old': datbins['lure']['old']['ROC'][selector].mean(0),
                'new': datbins['lure']['new']['ROC'][selector].mean(0)}
            targdatbins = {
                'old': datbins['targ']['old']['ROC'][selector].mean(0),
                'new': datbins['targ']['new']['ROC'][selector].mean(0)}
    if datbins is None:
        ax.plot(luredat['old'], targdat['old'], 'r-',
                linewidth=2., label='"old"')
        ax.plot(targdat['new'], luredat['new'], 'g--',
                linewidth=2., label='"new"')
        ax.plot(luredat['old'], targdat['old'], 'r^')
        ax.plot(targdat['new'], luredat['new'], 'gv')
    else:
        ax.plot(luredat['old'], targdat['old'], 'r-',
                linewidth=2., label='"old"')
        ax.plot(luredatbins['old'], targdatbins['old'], 'r^')
        ax.plot(targdat['new'], luredat['new'], 'g--',
                linewidth=2., label='"new"')
        ax.plot(targdatbins['new'], luredatbins['new'], 'gv')
    ax.plot([0, 1], [0, 1], 'k:', linewidth=2)
    ax.set_xlabel('$p($FA$)$')
    ax.set_ylabel('$p($HIT$)$')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))


def plot_areabars(ax, dat, selector=None):
    if selector is None:
        luredat = {'old': dat['lure']['old']['ROC'],
                   'new': dat['lure']['new']['ROC']}
        targdat = {'old': dat['targ']['old']['ROC'],
                   'new': dat['targ']['new']['ROC']}
    else:
        luredat = {'old': dat['lure']['old']['ROC'][selector],
                   'new': dat['lure']['new']['ROC'][selector]}
        targdat = {'old': dat['targ']['old']['ROC'][selector],
                   'new': dat['targ']['new']['ROC'][selector]}
    old_areas = []
    new_areas = []
    for lure, target in zip(luredat['old'], targdat['old']):
        old_areas.append(auc(lure, target))
    for lure, target in zip(dat['lure']['new']['ROC'],
                            dat['targ']['new']['ROC']):
        new_areas.append(auc(target, lure))
    ax.bar(-0.4, np.mean(old_areas), width=0.8, color='red', linewidth=2,
           edgecolor='black', label='old')
    ax.bar(0.6, np.mean(new_areas), width=0.8, color='green', linewidth=2,
           edgecolor='black', linestyle='dashed', label='new')
    ci95 = np.array([
        ((np.std(old_areas, ddof=1) / np.sqrt(len(old_areas) - 1)) *
         dists.t.ppf(0.975, len(old_areas) - 1)),
        ((np.std(new_areas, ddof=1) / np.sqrt(len(new_areas) - 1)) *
         dists.t.ppf(0.975, len(new_areas) - 1))])
    ax.errorbar([0, 1], [np.mean(old_areas), np.mean(new_areas)],
                yerr=ci95, fmt=None, elinewidth=2, ecolor='black')
    ax.set_xlim((-0.6, 1.6))
    ax.set_xticks([0, 1])
    ax.set_xticklabels(['"old"', '"new"'])
    ax.set_ylabel('Area under ROC')
    ax.set_ylim((0.5, 1))


def make_oldnew_roc_fig(distplots, filename=None, selector=None):
    fig = plt.figure(figsize=(8.8, 5))
    gs = gridspec.GridSpec(2, 20, left=0.08, right=0.99, bottom=0.095,
                           top=0.94, wspace=5, hspace=0.3)
    barpanel_plts = 4
    rocpanel_plts = 5
    ax = plt.subplot(gs[0, :barpanel_plts])
    plot_barpanel(ax, distplots['all']['conf'], 'new', selector=selector)
    ax.set_xlabel('confidence')
    ax.legend(loc='upper right', frameon=False, borderpad=0.)
    ax.set_title('"new"', size='xx-large')
    ax.set_ylabel('$p(x|$"new"$)$')
    ax.text(-0.4, 1.07, 'A', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[0, (barpanel_plts + 1):(barpanel_plts*2 + 1)])
    plot_barpanel(ax, distplots['all']['conf'], 'old', selector=selector)
    ax.set_xlabel('confidence')
    ax.set_title('"old"', size='xx-large')
    ax.set_ylabel('$p(x|$"old"$)$')
    ax = plt.subplot(
        gs[0, (barpanel_plts*2+2):(barpanel_plts*2+2+rocpanel_plts)],
        aspect='equal')
    plot_rocpanel(ax, distplots['all']['conf'], selector=selector)
    ax.legend(loc='lower right', frameon=False, borderpad=0.,
              handletextpad=0.2)
    ax.text(-0.3, 1.1, 'B', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[0, (barpanel_plts*2+3+rocpanel_plts):])
    plot_areabars(ax, distplots['all']['conf'], selector=selector)
    ax.text(-0.4, 1.07, 'C', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[1, :barpanel_plts])
    plot_barpanel(ax, distplots['bins']['rt'], 'new', selector=selector)
    ax.set_xlabel('RT strength')
    ax.legend(loc='upper right', frameon=False, borderpad=0.)
    ax.set_ylabel('$p(x|$"new"$)$')
    ax = plt.subplot(gs[1, (barpanel_plts + 1):(barpanel_plts*2 + 1)])
    plot_barpanel(ax, distplots['bins']['rt'], 'old', selector=selector)
    ax.set_xlabel('RT strength')
    ax.set_ylabel('$p(x|$"old"$)$')
    ax = plt.subplot(
        gs[1, (barpanel_plts*2+2):(barpanel_plts*2+2+rocpanel_plts)],
        aspect='equal')
    plot_rocpanel(
        ax, distplots['all']['rt'], distplots['bins']['rt'], selector=selector)
    ax.legend(
        loc='lower right', frameon=False, borderpad=0., handletextpad=0.2)
    ax = plt.subplot(gs[1, (barpanel_plts*2 + 3 + rocpanel_plts):])
    plot_areabars(ax, distplots['all']['rt'], selector=selector)
    if filename is not None:
        fig.savefig(filename)


def get_distplots_fullroc(resprates):
    resprates_fullroc = {'all': deepcopy(resprates[0]['indiv_resprates']),
                         'bins': deepcopy(resprates[1]['indiv_resprates'])}
    allbins = ['all', 'bins']
    dvs = ['conf', 'rt']
    distplots_fullroc = {}
    for ab in allbins:
        distplots_fullroc[ab] = {}
        for dv in dvs:
            distplots_fullroc[ab][dv] = {}
            for tl in ['targ', 'lure']:
                distplots_fullroc[ab][dv][tl] = {}
                for on in ['old', 'new']:
                    distplots_fullroc[ab][dv][tl][on] = {'dat': None,
                                                         'ROC': None}
            for ss in ['subj', 'sess']:
                distplots_fullroc[ab][dv][ss] = None
    for ab in distplots_fullroc:
        for dv in distplots_fullroc[ab]:
            distplots_fullroc[ab][dv]['subj'] = np.array(
                resprates_fullroc[ab][dv]['participant'])
            distplots_fullroc[ab][dv]['sess'] = np.array(
                resprates_fullroc[ab][dv]['session'])
            for targlure in ['targ', 'lure']:
                distplots_fullroc[ab][dv][targlure]['old']['dat'] = (
                    resprates_fullroc[ab][dv][targlure+'pres'][
                        :, np.int(resprates_fullroc[ab][dv][
                            targlure+'pres'].shape[1]/2.):])
                distplots_fullroc[ab][dv][targlure]['new']['dat'] = (
                    resprates_fullroc[ab][dv][targlure+'pres'][
                        :, :np.int(resprates_fullroc[ab][dv][
                            targlure+'pres'].shape[1]/2.)])
            # keep only those subjects/sessions that have both "old"
            # and "new" responses for both targets and lures:
            filter_indx = np.c_[
                distplots_fullroc[ab][dv]['targ']['old']['dat'].sum(1),
                distplots_fullroc[ab][dv]['targ']['new']['dat'].sum(1),
                distplots_fullroc[ab][dv]['lure']['old']['dat'].sum(1),
                distplots_fullroc[ab][dv]['lure']['new']['dat'].sum(1)].min(
                    1) > 0
            for subjsess in ['subj', 'sess']:
                distplots_fullroc[ab][dv][subjsess] = distplots_fullroc[
                    ab][dv][subjsess][filter_indx]
            for targlure in ['targ', 'lure']:
                distplots_fullroc[ab][dv][targlure]['old'][
                    'dat'] = distplots_fullroc[ab][dv][targlure][
                        'old']['dat'][filter_indx]
                distplots_fullroc[ab][dv][targlure]['new'][
                    'dat'] = distplots_fullroc[ab][dv][targlure][
                        'new']['dat'][filter_indx]
                distplots_fullroc[ab][dv][targlure]['new']['ROC'] = np.c_[
                    np.ones(distplots_fullroc[ab][dv][targlure]['new'][
                        'dat'].shape[0]),
                    1 - distplots_fullroc[ab][dv][
                        targlure]['new']['dat'].cumsum(1)]
                newsum = np.atleast_2d(distplots_fullroc[ab][dv][
                    targlure]['new']['dat'].sum(1)).T.repeat(distplots_fullroc[
                        ab][dv][targlure]['new']['dat'].shape[1], axis=1)
                distplots_fullroc[ab][dv][targlure]['old'][
                    'ROC'] = np.c_[
                        1-newsum-distplots_fullroc[ab][dv][targlure]['old'][
                            'dat'].cumsum(1),
                        np.zeros(distplots_fullroc[ab][dv][
                            targlure]['old']['dat'].shape[0])]
                # The zeros need to be added because the cumsum will be
                # different from zero depending on the bin size.
                for oldnew in distplots_fullroc[ab][dv][targlure]:
                    if (distplots_fullroc[ab][dv][targlure][oldnew][
                            'ROC'].max() > 1.0001):
                        raise ValueError('Max ROC = ' + str(distplots_fullroc[
                            ab][dv][targlure][oldnew]['ROC'].max()))
                        if (distplots_fullroc[ab][dv][targlure][oldnew][
                                'ROC'].min() < -0.0001):
                            raise ValueError('Min ROC = ' + str(
                                distplots_fullroc[
                                    ab][dv][targlure][oldnew]['ROC'].min()))
                            distplots_fullroc[ab][dv][targlure][oldnew]['ROC'][
                                distplots_fullroc[ab][dv][targlure][oldnew][
                                    'ROC'] > 1] = 1
                            distplots_fullroc[ab][dv][targlure][oldnew]['ROC'][
                                distplots_fullroc[ab][dv][targlure][oldnew][
                                    'ROC'] < 0] = 0
    return(distplots_fullroc)


def get_distplots_fullroc_subjmean(distplots_fullroc):
    distplots_fullroc_subjmean = {}
    for datpartition in distplots_fullroc:
        distplots_fullroc_subjmean[datpartition] = {}
        for allbins in distplots_fullroc[datpartition]:
            distplots_fullroc_subjmean[datpartition][allbins] = {}
            for dv in distplots_fullroc[datpartition][allbins]:
                distplots_fullroc_subjmean[datpartition][allbins][dv] = {}
                num_subj = len(np.unique(distplots_fullroc[
                    datpartition][allbins][dv]['subj']))
                distplots_fullroc_subjmean[
                    datpartition][allbins][dv]['subj'] = np.empty(
                        num_subj, 'S6')
                distplots_fullroc_subjmean[datpartition][allbins][dv][
                    'num_of_sessions'] = np.ones(num_subj, np.int) * -1
                for targlure in ['targ', 'lure']:
                    distplots_fullroc_subjmean[datpartition][allbins][
                        dv][targlure] = {}
                    for oldnew in distplots_fullroc[datpartition][allbins][
                                dv][targlure]:
                        distplots_fullroc_subjmean[datpartition][allbins][
                            dv][targlure][oldnew] = {}
                        for datroc in distplots_fullroc[datpartition][
                                allbins][dv][targlure][oldnew]:
                            array_shape = list(np.shape(distplots_fullroc[
                                datpartition][allbins][dv][
                                    targlure][oldnew][datroc]))
                            array_shape[0] = num_subj
                            distplots_fullroc_subjmean[datpartition][allbins][
                                dv][targlure][oldnew][datroc] = np.ones(
                                    array_shape, np.float128)*np.nan
                for s, subj in enumerate(np.unique(distplots_fullroc[
                        datpartition][allbins][dv]['subj'])):
                    subjfilt = distplots_fullroc[
                        datpartition][allbins][dv]['subj'] == subj
                    distplots_fullroc_subjmean[
                        datpartition][allbins][dv]['subj'][s] = subj
                    distplots_fullroc_subjmean[
                        datpartition][allbins][dv]['num_of_sessions'][s] = len(
                            np.unique(distplots_fullroc[
                                datpartition][allbins][dv]['sess'][subjfilt]))
                    for targlure in ['targ', 'lure']:
                        for oldnew in distplots_fullroc[datpartition][allbins][
                                dv][targlure]:
                            for datroc in distplots_fullroc[datpartition][
                                    allbins][dv][targlure][oldnew]:
                                distplots_fullroc_subjmean[datpartition][
                                    allbins][dv][targlure][oldnew][
                                        datroc][s] = distplots_fullroc[
                                            datpartition][allbins][dv][
                                                targlure][oldnew][datroc][
                                                    subjfilt].mean(0)
    return(distplots_fullroc_subjmean)


def plot_fullrocpanel(ax, dat, datbins=None, selector=None):
    if selector is None:
        dattoplot = {'lure': np.r_[dat['lure']['new']['ROC'].mean(0),
                                   dat['lure']['old']['ROC'].mean(0)],
                     'targ': np.r_[dat['targ']['new']['ROC'].mean(0),
                                   dat['targ']['old']['ROC'].mean(0)]}
        if datbins is not None:
            dattoplot_bins = {
                'lure': np.r_[datbins['lure']['new']['ROC'].mean(0),
                              datbins['lure']['old']['ROC'].mean(0)],
                'targ': np.r_[datbins['targ']['new']['ROC'].mean(0),
                              datbins['targ']['old']['ROC'].mean(0)]}
    else:
        dattoplot = {
            'lure': np.r_[dat['lure']['new']['ROC'][selector].mean(0),
                          dat['lure']['old']['ROC'][selector].mean(0)],
            'targ': np.r_[dat['targ']['new']['ROC'][selector].mean(0),
                          dat['targ']['old']['ROC'][selector].mean(0)]}
        if datbins is not None:
            dattoplot_bins = {
                'lure': np.r_[datbins['lure']['new']['ROC'][selector].mean(0),
                              datbins['lure']['old']['ROC'][selector].mean(0)],
                'targ': np.r_[datbins['targ']['new']['ROC'][selector].mean(0),
                              datbins['targ']['old']['ROC'][selector].mean(0)]}
    ax.plot([0, 1], [0, 1], 'k:', linewidth=2)
    ax.plot([0, dattoplot['lure'][(len(dattoplot['lure']) / 2) - 1]],
            [0, dattoplot['targ'][(len(dattoplot['targ']) / 2) - 1]],
            'k:', linewidth=2)
    ax.plot([dattoplot['lure'][(len(dattoplot['lure']) / 2) - 1], 1],
            [dattoplot['targ'][(len(dattoplot['targ']) / 2) - 1], 1],
            'k:', linewidth=2)
    if datbins is None:
        ax.plot(dattoplot['lure'][(len(dattoplot['lure']) / 2) - 1:],
                dattoplot['targ'][(len(dattoplot['targ']) / 2) - 1:],
                'r-', linewidth=2., label='"old"')
        ax.plot(dattoplot['lure'][:(len(dattoplot['lure']) / 2)],
                dattoplot['targ'][:(len(dattoplot['targ']) / 2)],
                'g--', linewidth=2., label='"new"')
        ax.plot(dattoplot['lure'][(len(dattoplot['lure']) / 2):],
                dattoplot['targ'][(len(dattoplot['targ']) / 2):], 'r^')
        ax.plot(dattoplot['lure'][:(len(dattoplot['lure']) / 2) - 1],
                dattoplot['targ'][:(len(dattoplot['targ']) / 2) - 1], 'gv')
    else:
        ax.plot(dattoplot['lure'][(len(dattoplot['lure']) / 2):],
                dattoplot['targ'][(len(dattoplot['targ']) / 2):],
                'r-', linewidth=2., label='"old"')
        ax.plot(dattoplot['lure'][:(len(dattoplot['lure']) / 2) + 1],
                dattoplot['targ'][:(len(dattoplot['targ']) / 2) + 1],
                'g--', linewidth=2., label='"new"')
        ax.plot(
            dattoplot_bins['lure'][
                (len(dattoplot_bins['lure']) / 2) + 1:],
            dattoplot_bins['targ'][
                (len(dattoplot_bins['targ']) / 2) + 1:], 'r^')
        ax.plot(
            dattoplot_bins['lure'][:(len(dattoplot_bins['lure']) / 2)],
            dattoplot_bins['targ'][:(len(dattoplot_bins['targ']) / 2)], 'gv')
    ax.plot(dattoplot['lure'][(len(dattoplot['lure']) / 2) - 1],
            dattoplot['targ'][(len(dattoplot['targ']) / 2) - 1], 'bD')
    ax.set_xlabel('$p($FA$)$')
    ax.set_ylabel('$p($HIT$)$')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))


def plot_fullbarpanel(ax, dat, oldnew, targcol='blue', lurecol='orange',
                      targhatch='/', lurehatch='\\', selector=None):
    if selector is None:
        targdat = dat['targ'][oldnew]['dat']
        luredat = dat['lure'][oldnew]['dat']
    else:
        targdat = dat['targ'][oldnew]['dat'][selector]
        luredat = dat['lure'][oldnew]['dat'][selector]
    if oldnew == 'old':
        ax.bar(np.arange(0.6, 5.6, 1), targdat.mean(0),
               alpha=0.3, hatch=targhatch, ec='black', fc=targcol)
        ax.bar(np.arange(0.6, 5.6, 1), targdat.mean(0),
               hatch=targhatch, ec='black', fc='none', linewidth=2.,
               linestyle='solid', label='target')
        ax.bar(np.arange(0.6, 5.6, 1), luredat.mean(0),
               alpha=0.3, hatch=lurehatch, ec='black', fc=lurecol)
        ax.bar(np.arange(0.6, 5.6, 1), luredat.mean(0),
               hatch=lurehatch, ec='black', fc='none', linewidth=2.,
               linestyle='dashed', label='lure')
        ax.set_xticks(np.arange(1, 6))
        ax.set_xticklabels(np.arange(1, 6))
    elif oldnew == 'new':
        ax.bar(np.arange(0.6, 5.6, 1), targdat[::-1].mean(0),
               alpha=0.3, hatch=targhatch, ec='black', fc=targcol)
        ax.bar(np.arange(0.6, 5.6, 1), targdat[::-1].mean(0),
               hatch=targhatch, ec='black', fc='none', linewidth=2.,
               linestyle='solid', label='target')
        ax.bar(np.arange(0.6, 5.6, 1), luredat[::-1].mean(0),
               alpha=0.3, hatch=lurehatch, ec='black', fc=lurecol)
        ax.bar(np.arange(0.6, 5.6, 1), luredat[::-1].mean(0),
               hatch=lurehatch, ec='black', fc='none', linewidth=2.,
               linestyle='dashed', label='lure')
        ax.set_xticks(np.arange(1, 6))
        ax.set_xticklabels(np.arange(5, 0, -1))
    else:
        raise ValueError('Invalid oldnew value: ' + str(oldnew))
    ax.set_ylim((0, 1))
    ax.set_xlim((0.4, 5.6))
    ax.set_xlabel('$p(x)$')


def plot_fullareabar(ax, dat, selector=None):
    areas = []
    if selector is None:
        luredat = np.c_[dat['lure']['new']['ROC'], dat['lure']['old']['ROC']]
        targdat = np.c_[dat['targ']['new']['ROC'], dat['targ']['old']['ROC']]
        # add maker for area under random ROC
        classification_point = (
            dat['lure']['new']['ROC'].mean(0)[-1],
            dat['targ']['new']['ROC'].mean(0)[-1])
    else:
        luredat = np.c_[dat['lure']['new']['ROC'][selector],
                        dat['lure']['old']['ROC'][selector]]
        targdat = np.c_[dat['targ']['new']['ROC'][selector],
                        dat['targ']['old']['ROC'][selector]]
        # add maker for area under random ROC
        classification_point = (
            dat['lure']['new']['ROC'][selector].mean(0)[-1],
            dat['targ']['new']['ROC'][selector].mean(0)[-1])
    for lure, target in zip(luredat, targdat):
        # Sometimes rounding errors can produce very small negative
        # values.  First check that negative values are small enough
        # to be neglegible:
        if np.min(lure) < -0.1e-10:
            raise ValueError("Negative Lures! " + str(lure))
        elif np.min(target) < -0.1e-10:
            raise ValueError("Negative Targets! " + str(target))
        # Set any neglegibly negative value to zero (otherwise auc
        # complains):
        lure[lure < 0] = 0
        target[target < 0] = 0
        areas.append(auc(lure, target, reorder=True))
    ax.bar([-0.4], [np.mean(areas)], width=0.8, color='white', linewidth=2,
           edgecolor='black')
    ax.bar([-0.4], [np.mean(areas)], width=0.8, color='blue', alpha=0.2)
    ci95 = np.array([((np.std(areas, ddof=1) / np.sqrt(len(areas) - 1)) *
                      dists.t.ppf(0.975, len(areas) - 1))])
    ax.errorbar([0], [np.mean(areas)], yerr=ci95, fmt=None, elinewidth=2,
                ecolor='black')
    ax.set_xlim((-0.6, 0.6))
    ax.set_xticks([0])
    ax.set_xticklabels(['all\nresponses'])
    ax.set_ylabel('Area under ROC')
    ax.set_ylim((0.5, 1))
    # base of the triangle is main diagonal:
    base = np.sqrt(2)
    # the left side of the triangle spanned by random roc and main diagonal:
    left_side = np.sqrt(
        classification_point[0]**2 + classification_point[1]**2)
    # the angle in the left corner of that triangle is calculated by
    # getting the angle spanned between the left leg of the random ROC
    # and the x-axis and subtracting 45 degree (angle of the main
    # diagonal from it:
    left_angle = np.arcsin(
        classification_point[1] / left_side) - np.deg2rad(45)
    # the height of the triangle (for area calculation):
    height = np.sin(left_angle) * left_side
    # the area in the triangle formed by the random ROC and the main
    # diagonal plus the area under the main diagonal:
    area_under_randomROC = ((base * height) / 2) + 0.5
    ax.set_ylim(np.round(area_under_randomROC, 2), 1)


def make_roc_fig(distplots_fullroc, filename=None, selector=None):
    fig = plt.figure(figsize=(8, 5))
    gs = gridspec.GridSpec(2, 10, left=0.08, right=0.98, bottom=0.1, top=0.92,
                           wspace=0.05, hspace=0.4)
    ax = plt.subplot(gs[0, :2])
    plot_fullbarpanel(
        ax, distplots_fullroc['all']['conf'], 'new', selector=selector)
    ax.set_xlabel('confidence')
    ax.legend(loc='upper right', frameon=False, borderpad=0.)
    ax.set_title('"new"', size='xx-large')
    ax.set_ylabel('$p(x)$')
    ax.text(-0.35, 1.1, 'A', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[0, 2:4])
    plot_fullbarpanel(
        ax, distplots_fullroc['all']['conf'], 'old', selector=selector)
    ax.set_yticklabels([])
    ax.set_xlabel('confidence')
    ax.set_title('"old"', size='xx-large')
    ax = plt.subplot(gs[0, 4:9], aspect='equal')
    plot_fullrocpanel(ax, distplots_fullroc['all']['conf'], selector=selector)
    ax.legend(
        loc='lower right', frameon=False, borderpad=0., handletextpad=0.2)
    ax.text(-0.3, 1.1, 'B', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[0, 9])
    plot_fullareabar(ax, distplots_fullroc['all']['conf'], selector=selector)
    ax.text(-1, 1.1, 'C', horizontalalignment='center',
            verticalalignment='center', clip_on=False,
            transform=ax.transAxes, size='xx-large', weight='bold')
    ax = plt.subplot(gs[1, :2])
    plot_fullbarpanel(
        ax, distplots_fullroc['bins']['rt'], 'new', selector=selector)
    ax.set_xlabel('RT strength')
    ax.legend(loc='upper right', frameon=False, borderpad=0.)
    ax.set_ylabel('$p(x)$')
    ax = plt.subplot(gs[1, 2:4])
    plot_fullbarpanel(
        ax, distplots_fullroc['bins']['rt'], 'old', selector=selector)
    ax.set_yticklabels([])
    ax.set_xlabel('RT strength')
    ax = plt.subplot(gs[1, 4:9], aspect='equal')
    plot_fullrocpanel(ax, distplots_fullroc['all']['rt'],
                      distplots_fullroc['bins']['rt'], selector=selector)
    ax.legend(
        loc='lower right', frameon=False, borderpad=0., handletextpad=0.2)
    ax = plt.subplot(gs[1, 9])
    plot_fullareabar(ax, distplots_fullroc['all']['rt'], selector=selector)
    if filename is not None:
        fig.savefig(filename)


def get_hitfadiff_ratio(events):
    # Compute ratio of (HITs-FAs) for fastest over slowest RTs for each
    # subj/session:
    hitfadiff_ratio = np.ones(len(events), np.float) * np.nan
    hits = np.ones(len(events), np.float) * np.nan
    fas = np.ones(len(events), np.float) * np.nan
    participants = np.zeros(len(events), '|S8')
    sessions = np.zeros(len(events), np.int)
    bins = 2
    hitfadiffs = np.ones((bins, len(events)), np.float) * np.nan
    counter = {'accepted': 0, 'rejected': 0}
    for subj in np.unique(events['participant']):
        subjfilt = events['participant'] == subj
        for sess in np.unique(
                events[events['participant'] == subj]['session']):
            subjsessfilt = (subjfilt &
                            (events['session'] == sess))
            tmpevs = events[subjsessfilt]
            rts_sorted = np.sort(tmpevs['recog_rt'])
            rt_thresholds = [rts_sorted[
                i * (np.int(np.ceil(len(rts_sorted) / np.float(bins))))]
                             for i in range(bins)]
            rt_thresholds.append(rts_sorted.max())
            event_bins = [tmpevs[
                ((tmpevs['recog_rt'] >= rt_thresholds[i]) &
                 (tmpevs['recog_rt'] < rt_thresholds[i+1]))]
                          for i in range(len(rt_thresholds)-1)]
            hit_bins = np.array([
                (np.sum((evb['type'] == 'RECOG_TARGET') &
                        (evb['recog_resp'] == 1)) /
                 np.float(np.sum(evb['type'] == 'RECOG_TARGET')))
                for evb in event_bins])
            hit_bins[~np.isfinite(hit_bins)] = 0.
            fa_bins = np.array([
                (np.sum((evb['type'] == 'RECOG_LURE') &
                        (evb['recog_resp'] == 1)) /
                 np.float(np.sum(evb['type'] == 'RECOG_LURE')))
                for evb in event_bins])
            fa_bins[~np.isfinite(fa_bins)] = 0.
            hitfadiff = hit_bins - fa_bins
            for i in range(len(hitfadiff)):
                hitfadiffs[i][subjsessfilt] = hitfadiff[i]
            participants[subjsessfilt] = subj
            sessions[subjsessfilt] = sess
            hits[subjsessfilt] = np.sum(
                ((tmpevs['type'] == 'RECOG_TARGET') &
                 (tmpevs['recog_resp'] == 1)) /
                np.float(np.sum(tmpevs['type'] == 'RECOG_TARGET')))
            fas[subjsessfilt] = np.sum(
                ((tmpevs['type'] == 'RECOG_LURE') &
                 (tmpevs['recog_resp'] == 1)) /
                np.float(np.sum(tmpevs['type'] == 'RECOG_LURE')))
            if (np.min(hitfadiff) < 0.2) or (np.min(hitfadiff) > 0.9):
                hitfadiff_ratio[subjsessfilt] = np.nan
                counter['rejected'] += 1
            else:
                hitfadiff_ratio[subjsessfilt] = hitfadiff[0] / hitfadiff[-1]
                counter['accepted'] += 1
    hitfadiffs = np.array(hitfadiffs)
    hits = np.array(hits)
    fas = np.array(fas)
    participants = np.array(participants)
    sessions = np.array(sessions)
    return(hitfadiff_ratio, hitfadiffs, counter, hits, fas,
           participants, sessions)


events = {'all_recunrec': data_recunrecfilt,
          'all_targlure': data_targlurefilt,
          'recalled': data_recunrecfilt[
              (data_recunrecfilt['type'] != 'RECOG_TARGET') |
              (data_recunrecfilt['recalled'] == 'yes')],
          'unrecalled': data_recunrecfilt[
              (data_recunrecfilt['type'] != 'RECOG_TARGET') |
              (data_recunrecfilt['recalled'] == 'no')]}

rtconfs = {}
for evcat in events:
    rtconfs[evcat] = {
        'recog_rt': {'hit': [], 'miss': [], 'fa': [], 'cr': []},
        'recog_conf': {'hit': [], 'miss': [], 'fa': [], 'cr': []}}
    targfilt = events[evcat]['type'] == 'RECOG_TARGET'
    lurefilt = events[evcat]['type'] == 'RECOG_LURE'
    oldfilt = events[evcat]['recog_resp'] == 'old'
    newfilt = events[evcat]['recog_resp'] == 'new'
    for subj in np.unique(events[evcat]['participant']):
        subjfilt = events[evcat]['participant'] == subj
        for sess in np.unique(events[evcat][
                subjfilt]['session']):
            subjsessfilt = subjfilt & (events[evcat]['session'] == sess)
            for dv in rtconfs[evcat]:
                rtconfs[evcat][dv]['hit'].append(
                    events[evcat][subjsessfilt & targfilt & oldfilt][dv])
                rtconfs[evcat][dv]['miss'].append(
                    events[evcat][subjsessfilt & targfilt & newfilt][dv])
                rtconfs[evcat][dv]['fa'].append(  # false alarm
                    events[evcat][subjsessfilt & lurefilt & oldfilt][dv])
                rtconfs[evcat][dv]['cr'].append(  # correct rejection
                    events[evcat][subjsessfilt & lurefilt & newfilt][dv])

rtconfs_centraltendency = {}
for evcat in rtconfs:
    rtconfs_centraltendency[evcat] = {}
    for dv in rtconfs[evcat]:
        rtconfs_centraltendency[evcat][dv] = {}
        for measure in rtconfs[evcat][dv]:
            rtconfs_centraltendency[evcat][dv][measure] = {
                'mean': np.array([np.mean(dat)
                                  for dat in rtconfs[evcat][dv][measure]]),
                'median': np.array([np.median(dat)
                                    for dat in rtconfs[evcat][dv][measure]]),
                'n': np.array([len(dat)
                               for dat in rtconfs[evcat][dv][measure]])}

resprates_dict_agg = {}
resprates_dict_agg_10 = {}
distplots = {}
distplots_fullroc = {}
hitfadiff_ratio = {}
hitfadiffs = {}
counter = {}
hits = {}
fas = {}
hitfasubj = {}
hitfasess = {}
for evcat in events:
    resprates_dict_agg[evcat], resprates_dict_agg_10[
        evcat] = get_resprates(events[evcat])
for evcat in events:
    distplots[evcat], filt_indx = get_distplots((resprates_dict_agg[evcat],
                                                 resprates_dict_agg_10[evcat]))
    if filt_indx.min() < 1:
        print evcat, filt_indx.sum(), len(filt_indx)
    distplots_fullroc[evcat] = get_distplots_fullroc(
        (resprates_dict_agg[evcat], resprates_dict_agg_10[evcat]))
    (hitfadiff_ratio[evcat], hitfadiffs[evcat], counter[evcat], hits[evcat],
     fas[evcat], hitfasubj[evcat], hitfasess[evcat]) = get_hitfadiff_ratio(
         events[evcat])


distplots_fullroc_subjmean = get_distplots_subjmean(distplots_fullroc)
make_roc_fig(distplots_fullroc_subjmean['all_targlure'],
             filename='./figs/roc_subjmean_all_targlure_both.pdf')

distplots_subjmean = get_distplots_subjmean(distplots)
make_oldnew_roc_fig(distplots_subjmean['all_targlure'],
                    filename='./figs/roc_subjmean_oldnew_targlure_both.pdf')


def get_selhitsfas(dat, hits, fas, hitfasubj, hitfasess, selector=None):
    selected_hits = []
    selected_fas = []
    if selector is None:
        subjects = np.unique(dat['subj'])
    else:
        subjects = np.unique(dat['subj'][selector])
    for subj in subjects:
        if selector is None:
            sessions = np.unique(dat['sess'][dat['subj'] == subj])
        else:
            sessions = np.unique(
                dat['sess'][selector][dat['subj'][selector] == subj])
        for sess in sessions:
            tmphit = np.unique(
                hits[(hitfasubj == subj) & (hitfasess == sess)])
            if len(tmphit) > 1:
                raise ValueError('Non unique hits: ' + str(tmphit))
            else:
                tmphit = tmphit[0]
            tmpfa = np.unique(fas[(hitfasubj == subj) & (hitfasess == sess)])
            if len(tmpfa) > 1:
                raise ValueError('Non unique fas: ' + str(tmpfa))
            else:
                tmpfa = tmpfa[0]
            selected_hits.append(tmphit)
            selected_fas.append(tmpfa)
    selected_hits = np.array(selected_hits)
    selected_fas = np.array(selected_fas)
    return(selected_hits, selected_fas)


def get_oldnew_areas(dat, selector=None):
    old_areas = []
    new_areas = []
    if selector is None:
        luredat = {'old': dat['lure']['old']['ROC'],
                   'new': dat['lure']['new']['ROC']}
        targdat = {'old': dat['targ']['old']['ROC'],
                   'new': dat['targ']['new']['ROC']}
    else:
        luredat = {'old': dat['lure']['old']['ROC'][selector],
                   'new': dat['lure']['new']['ROC'][selector]}
        targdat = {'old': dat['targ']['old']['ROC'][selector],
                   'new': dat['targ']['new']['ROC'][selector]}
    for lure, target in zip(luredat['old'], targdat['old']):
        old_areas.append(auc(lure, target))
    for lure, target in zip(luredat['new'], targdat['new']):
        new_areas.append(auc(target, lure))
    return np.array(old_areas), np.array(new_areas)


conf_oldnew_areas = {}
rt_oldnew_areas = {}
oldnew_areas = {}
for evcat in events:
    conf_oldnew_areas[evcat] = get_oldnew_areas(
        distplots[evcat]['all']['conf'])
    rt_oldnew_areas[evcat] = get_oldnew_areas(
        distplots[evcat]['all']['rt'])
    oldnew_areas[evcat] = {
        'conf': {
            'old': conf_oldnew_areas[evcat][0],
            'new': conf_oldnew_areas[evcat][1]},
        'rt': {
            'old': rt_oldnew_areas[evcat][0],
            'new': rt_oldnew_areas[evcat][1]}}


oldnew_areas_subj = {}
for evcat in oldnew_areas:
    oldnew_areas_subj[evcat] = {'conf': {}, 'rt': {}}
    subj_array = np.array(
        resprates_dict_agg[evcat]['indiv_resprates']['conf']['participant'])
    for dv in oldnew_areas[evcat]:
        for oldnew in oldnew_areas[evcat][dv]:
            oldnew_areas_subj[evcat][dv][oldnew] = np.array(
                [np.mean(oldnew_areas[evcat][dv][oldnew][subj_array == subj])
                 for subj in np.unique(subj_array)])


def get_areas(dat, selector=None):
    areas = []
    if selector is None:
        luredat = np.c_[dat['lure']['new']['ROC'], dat['lure']['old']['ROC']]
        targdat = np.c_[dat['targ']['new']['ROC'], dat['targ']['old']['ROC']]
    else:
        luredat = np.c_[dat['lure']['new']['ROC'][selector],
                        dat['lure']['old']['ROC'][selector]]
        targdat = np.c_[dat['targ']['new']['ROC'][selector],
                        dat['targ']['old']['ROC'][selector]]
    for lure, target in zip(luredat, targdat):
        if np.min(lure) < -0.1e-10:
            raise ValueError("Negative Lures! " + str(lure))
        elif np.min(target) < -0.1e-10:
            raise ValueError("Negative Targets! " + str(target))
        # Set any neglegibly negative value to zero (otherwise auc
        # complains):
        lure[lure < 0] = 0
        target[target < 0] = 0
        areas.append(auc(lure, target, reorder=True))
    return(np.array(areas))

regression_evcats = {}
areas_evcats = {}
areas_evcats_subj = {}
for evcat in events:
    subj_array = np.array(
        resprates_dict_agg[evcat]['indiv_resprates']['conf']['participant'])
    areas_evcats[evcat] = {'all': {}, 'residual': {}}
    areas_evcats_subj[evcat] = {'all': {}, 'residual': {}}
    areas_evcats[evcat]['all'] = {'conf': {}, 'rt': {},
                                  'old': {'conf': {}, 'rt': {}},
                                  'new': {'conf': {}, 'rt': {}}}
    areas_evcats_subj[evcat]['all'] = {'conf': {}, 'rt': {},
                                       'old': {'conf': {}, 'rt': {}},
                                       'new': {'conf': {}, 'rt': {}}}
    areas_evcats[evcat]['all']['conf'] = get_areas(
        distplots_fullroc[evcat]['all']['conf'])
    areas_evcats[evcat]['all']['rt'] = get_areas(
        distplots_fullroc[evcat]['all']['rt'])
    areas_evcats[evcat]['all']['old']['conf'], areas_evcats[evcat]['all'][
        'new']['conf'] = get_oldnew_areas(distplots[evcat]['all']['conf'])
    areas_evcats[evcat]['all']['old']['rt'], areas_evcats[evcat]['all'][
        'new']['rt'] = get_oldnew_areas(distplots[evcat]['all']['rt'])
    regression_evcats[evcat] = {'conf': {}, 'rt': {}}
    (regression_evcats[evcat]['hits'],
     regression_evcats[evcat]['fas']) = get_selhitsfas(
         dat=distplots_fullroc[evcat]['all']['rt'],
         hits=hits[evcat], fas=fas[evcat], hitfasubj=hitfasubj[evcat],
         hitfasess=hitfasess[evcat])
    (regression_evcats[evcat]['conf']['slope'],
     regression_evcats[evcat]['conf']['intercept'],
     regression_evcats[evcat]['conf']['r'],
     regression_evcats[evcat]['conf']['p'],
     regression_evcats[evcat]['conf']['SE']) = stats.linregress(
         regression_evcats[evcat]['hits'] - regression_evcats[evcat]['fas'],
         np.float64(areas_evcats[evcat]['all']['conf']))
    (regression_evcats[evcat]['rt']['slope'],
     regression_evcats[evcat]['rt']['intercept'],
     regression_evcats[evcat]['rt']['r'],
     regression_evcats[evcat]['rt']['p'],
     regression_evcats[evcat]['rt']['SE']) = stats.linregress(
         regression_evcats[evcat]['hits'] - regression_evcats[evcat]['fas'],
         np.float64(areas_evcats[evcat]['all']['rt']))
    areas_evcats[evcat]['residual'] = {'conf': {}, 'rt': {}}
    areas_evcats[evcat]['residual']['conf'] = areas_evcats[evcat]['all'][
        'conf']-((regression_evcats[evcat]['hits'] -
                  regression_evcats[evcat]['fas']) *
                 regression_evcats[evcat]['conf']['slope'] +
                 regression_evcats[evcat]['conf']['intercept'])
    areas_evcats[evcat]['residual']['rt'] = areas_evcats[evcat]['all'][
        'rt']-((regression_evcats[evcat]['hits'] -
                regression_evcats[evcat]['fas']) *
               regression_evcats[evcat]['rt']['slope'] +
               regression_evcats[evcat]['rt']['intercept'])
    for allres in areas_evcats[evcat]:
        for key in areas_evcats[evcat][allres]:
            if isinstance(areas_evcats[evcat][allres][key], dict):
                for key2 in areas_evcats[evcat][allres][key]:
                    areas_evcats_subj[evcat][allres][key][key2] = np.array(
                        [np.mean(areas_evcats[evcat][allres][key][key2][
                            subj_array == subj])
                         for subj in np.unique(subj_array)])
            else:
                areas_evcats_subj[evcat][allres][key] = np.array(
                    [np.mean(areas_evcats[evcat][allres][key][
                        subj_array == subj])
                     for subj in np.unique(subj_array)])


def make_corr_fig(selected_hits, selected_fas,
                  conf_areas, rt_areas, filename=None):
    areas_diff = conf_areas - rt_areas
    fig = plt.figure(figsize=(3, 6))
    grid = gridspec.GridSpec(nrows=40, ncols=1, wspace=0.0, hspace=50.0,
                             bottom=0.1, top=0.97, left=0.3, right=0.93)
    ax = plt.subplot(grid[:10, 0])
    ax.plot(selected_hits - selected_fas, conf_areas, 'k.', alpha=0.4)
    ax.set_xlabel(r'HIT$-$FA', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{C}$', size='large')
    ax.set_xticks(np.arange(0.0, 1.01, 0.2))
    ax.set_yticks(np.arange(0.5, 1.01, 0.1))
    ax.set_xticklabels(['0', '.2', '.4', '.6', '.8', '1'], size='large')
    ax.set_yticklabels(np.arange(0.5, 1.01, 0.1), size='large')
    ax.text(0.1, 0.8, 'A', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.2, r'$\mathsf{r=}$' + str(np.round(np.corrcoef(
        selected_hits - selected_fas, conf_areas)[0, 1], 2))[-3:],
        horizontalalignment='right', verticalalignment='center',
        transform=ax.transAxes, size='large')
    ax.set_xlim((-0.05, 1.0))
    ax.set_ylim((0.45, 1.0))
    SE_r_hitFAaucC = np.sqrt((
        1 - np.corrcoef(selected_hits - selected_fas, conf_areas)[0, 1]) /
        (len(selected_hits) - 2))
    t_r_hitFAaucC = np.corrcoef(selected_hits - selected_fas,
                                conf_areas)[0, 1] / SE_r_hitFAaucC
    df_r_hitFA = len(selected_hits) - 2
    p_hitFA_aucC = scipy.stats.t.sf(np.float(t_r_hitFAaucC),
                                    df_r_hitFA)
    ax = plt.subplot(grid[15:25, 0])
    ax.plot(selected_hits - selected_fas, rt_areas, 'k.', alpha=0.4)
    ax.set_xlabel(r'HIT$-$FA', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{L}$', size='large')
    ax.set_xticks(np.arange(0.0, 1.01, 0.2))
    ax.set_yticks(np.arange(0.5, 1.01, 0.1))
    ax.set_xticklabels(['0', '.2', '.4', '.6', '.8', '1'], size='large')
    ax.set_yticklabels(np.arange(0.5, 1.01, 0.1), size='large')
    ax.text(0.1, 0.8, 'B', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.2, r'$\mathsf{r=}$' + str(np.round(np.corrcoef(
        selected_hits - selected_fas, rt_areas)[0, 1], 2))[-3:],
        horizontalalignment='right', verticalalignment='center',
        transform=ax.transAxes, size='large')
    ax.set_xlim((-0.05, 1.0))
    ax.set_ylim((0.45, 1.0))
    SE_r_hitFAaucL = np.sqrt((
        1-np.corrcoef(selected_hits - selected_fas,
                      rt_areas)[0, 1]) / (len(selected_hits) - 2))
    t_r_hitFAaucL = np.corrcoef(selected_hits - selected_fas,
                                rt_areas)[0, 1] / SE_r_hitFAaucL
    p_hitFA_aucL = scipy.stats.t.sf(np.float(t_r_hitFAaucL), df_r_hitFA)

    ax = plt.subplot(grid[30:, 0])
    ax.plot(selected_hits - selected_fas, np.abs(areas_diff), 'k.', alpha=0.4)
    ax.set_xlabel(r'HIT$-$FA', size='large')
    ax.set_ylabel(r'|AUC$_\mathsf{C}-$AUC$_\mathsf{L}$|', size='large')
    ax.set_yticks(np.arange(0, 0.19, 0.04))
    ax.set_yticklabels(['.00', '.04', '.08', '.12', '.16'], size='large')
    ax.set_xticks(np.arange(0.0, 1.01, 0.2))
    ax.set_xticklabels(['0', '.2', '.4', '.6', '.8', '1'], size='large')
    ax.text(0.1, 0.8, 'C', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.83, 0.9, r'$\mathsf{r=}$',
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')
    ax.text(0.98, 0.7, r'$\mathsf{-}$' + str(np.round(np.corrcoef(
        selected_hits - selected_fas, np.abs(areas_diff))[0, 1], 2))[-3:],
        horizontalalignment='right', verticalalignment='center',
        transform=ax.transAxes, size='large')
    ax.set_xlim((-0.05, 1.0))
    if filename is not None:
        fig.savefig(filename)


def make_corr_aucs_fig(conf_areas, rt_areas, oldnew_areas, filename=None):
    panellabel_coords = (0.15, 0.85)
    corrtext_coords = (0.95, 0.1)
    figsize = (2.5, 7)
    fig = plt.figure(figsize=figsize)
    ax = plt.subplot(3, 1, 1, aspect='equal')
    ax.plot(conf_areas, rt_areas, 'k.', alpha=0.4)
    ax.set_xlabel(r'AUC$_\mathsf{C}$', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{L}$', size='large')
    ax.plot([0.4, 1], [0.4, 1], 'k-')
    ax.set_xlim((0.4, 1))
    ax.set_ylim((0.4, 1))
    ax.set_xticks(np.arange(0.5, 1, 0.2))
    ax.set_yticks(np.arange(0.5, 1, 0.2))
    ax.set_xticklabels(['.5', '.7', '.9'], size='x-large')
    ax.set_yticklabels(['.5', '.7', '.9'], size='x-large')
    ax.text(panellabel_coords[0], panellabel_coords[1], 'A',
            horizontalalignment='center', verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(corrtext_coords[0], corrtext_coords[1], r'$\mathsf{r=}$' +
            "{:.2f}".format(np.corrcoef(conf_areas, rt_areas)[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')
    ax = plt.subplot(3, 1, 2, aspect='equal')
    ax.plot(oldnew_areas['conf']['old'], oldnew_areas['rt']['old'],
            'k.', alpha=0.4)
    ax.set_xlabel(r'AUC$_\mathsf{C}$ ("O")', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{L}$ ("O")', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xticks(np.arange(0.2, 1, 0.2))
    ax.set_yticks(np.arange(0.2, 1, 0.2))
    ax.set_xticklabels(['.2', '.4', '.6', '.8'], size='large')
    ax.set_yticklabels(['.2', '.4', '.6', '.8'], size='large')
    ax.text(panellabel_coords[0], panellabel_coords[1], 'B',
            horizontalalignment='center',
            verticalalignment='center', transform=ax.transAxes,
            size='large', weight='bold')
    ax.text(corrtext_coords[0], corrtext_coords[1], r'$\mathsf{r=}$' +
            "{:.2f}".format(np.corrcoef(oldnew_areas['conf']['old'],
                                        oldnew_areas['rt']['old'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')
    ax = plt.subplot(3, 1, 3, aspect='equal')
    ax.plot(oldnew_areas['conf']['new'], oldnew_areas['rt']['new'],
            'k.', alpha=0.4)
    ax.set_xlabel(r'AUC$_\mathsf{C}$ ("N")', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{L}$ ("N")', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xticks(np.arange(0.2, 1, 0.2))
    ax.set_yticks(np.arange(0.2, 1, 0.2))
    ax.set_xticklabels(['.2', '.4', '.6', '.8'], size='large')
    ax.set_yticklabels(['.2', '.4', '.6', '.8'], size='large')
    ax.text(panellabel_coords[0], panellabel_coords[1], 'C',
            horizontalalignment='center', verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(corrtext_coords[0], corrtext_coords[1], r'$\mathsf{r=}$' +
            "{:.2f}".format(np.corrcoef(oldnew_areas['conf']['new'],
                                        oldnew_areas['rt']['new'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')
    plt.subplots_adjust(left=0.26, right=0.99, top=0.99, bottom=0.09,
                        wspace=0, hspace=0.4)
    fig.set_size_inches(figsize)
    if filename is not None:
        fig.savefig(filename)


def make_corr_aucs_fig_oldnew(oldnew_areas, filename=None):
    figsize = (5, 2.5)
    fig = plt.figure(figsize=figsize)
    ax = plt.subplot(1, 2, 1, aspect='equal')
    ax.plot(oldnew_areas['conf']['old'],
            oldnew_areas['conf']['new'], 'k.', alpha=0.4)
    ax.set_xlabel(r'AUC$_\mathsf{C}$ ("O")', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{C}$ ("N")', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))
    ax.set_xticks(np.arange(0.1, 1, 0.2))
    ax.set_yticks(np.arange(0.1, 1, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.text(0.15, 0.85, 'A', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.01, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['conf']['old'],
        oldnew_areas['conf']['new'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='bottom',
            transform=ax.transAxes, size='large')
    ax = plt.subplot(1, 2, 2, aspect='equal')
    ax.plot(oldnew_areas['rt']['old'],
            oldnew_areas['rt']['new'], 'k.', alpha=0.4)
    ax.set_xlabel(r'AUC$_\mathsf{L}$ ("O")', size='large')
    ax.set_ylabel(r'AUC$_\mathsf{L}$ ("N")', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))
    ax.set_xticks(np.arange(0.1, 1, 0.2))
    ax.set_yticks(np.arange(0.1, 1, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.text(0.15, 0.85, 'B', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.01, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['rt']['old'],
        oldnew_areas['rt']['new'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='bottom',
            transform=ax.transAxes, size='large')
    plt.subplots_adjust(left=0.13, right=0.99, top=0.999, bottom=0.22,
                        wspace=0.5, hspace=0)
    fig.set_size_inches(figsize)
    if filename is not None:
        fig.savefig(filename)


def make_corr_aucs_fig_recalled_unrecalled(oldnew_areas, filename=None):
    fig = plt.figure(figsize=(4.2, 4.35))
    ax = plt.subplot(2, 2, 1, aspect='equal')
    ax.plot(oldnew_areas['recalled']['conf']['old'],
            oldnew_areas['unrecalled']['conf']['old'], 'k.', alpha=0.4)
    ax.set_ylabel('confidence\nunrecalled', size='large')
    ax.set_title('"old"', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((-.15, 1.15))
    ax.set_ylim((-.15, 1.15))
    ax.set_xticks(np.arange(0.1, 1.0, 0.2))
    ax.set_yticks(np.arange(0.1, 1.0, 0.2))
    ax.set_xticklabels(['', '', '', '', '', ], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9', ], size='large')
    ax.text(0.15, 0.85, 'A', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.1, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['recalled']['conf']['old'],
        oldnew_areas['unrecalled']['conf']['old'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')
    ax = plt.subplot(2, 2, 2, aspect='equal')
    ax.plot(oldnew_areas['recalled']['conf']['new'],
            oldnew_areas['unrecalled']['conf']['new'], 'k.', alpha=0.4)
    ax.set_title('"new"', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((-.15, 1.15))
    ax.set_ylim((-.15, 1.15))
    ax.set_xticks(np.arange(0.1, 1.0, 0.2))
    ax.set_yticks(np.arange(0.1, 1.0, 0.2))
    ax.set_xticklabels(['', '', '', '', '', ], size='large')
    ax.set_yticklabels(['', '', '', '', '', ], size='large')
    ax.text(0.15, 0.85, 'B', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.1, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['recalled']['conf']['new'],
        oldnew_areas['unrecalled']['conf']['new'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')

    ax = plt.subplot(2, 2, 3, aspect='equal')
    ax.plot(oldnew_areas['recalled']['rt']['old'],
            oldnew_areas['unrecalled']['rt']['old'], 'k.', alpha=0.4)
    ax.set_xlabel('recalled', size='large')
    ax.set_ylabel('latency\nunrecalled', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((-.15, 1.15))
    ax.set_ylim((-.15, 1.15))
    ax.set_xticks(np.arange(0.1, 1.0, 0.2))
    ax.set_yticks(np.arange(0.1, 1.0, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9', ], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9', ], size='large')
    ax.text(0.15, 0.85, 'C', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.1, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['recalled']['rt']['old'],
        oldnew_areas['unrecalled']['rt']['old'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')

    ax = plt.subplot(2, 2, 4, aspect='equal')
    ax.plot(oldnew_areas['recalled']['rt']['new'],
            oldnew_areas['unrecalled']['rt']['new'], 'k.', alpha=0.4)
    ax.set_xlabel('recalled', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((-.15, 1.15))
    ax.set_ylim((-.15, 1.15))
    ax.set_xticks(np.arange(0.1, 1.0, 0.2))
    ax.set_yticks(np.arange(0.1, 1.0, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9', ], size='large')
    ax.set_yticklabels(['', '', '', '', '', ], size='large')
    ax.text(0.15, 0.85, 'D', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.1, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        oldnew_areas['recalled']['rt']['new'],
        oldnew_areas['unrecalled']['rt']['new'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='center',
            transform=ax.transAxes, size='large')

    plt.subplots_adjust(left=0.23, right=0.999, top=0.92, bottom=0.15,
                        wspace=0.05, hspace=0.05)
    if filename is not None:
        fig.savefig(filename)


make_corr_aucs_fig_recalled_unrecalled(
    oldnew_areas_subj, './figs/correlations_auc_recalled_unrecalled_subj.pdf')


def make_corr_aucs_fig_recalled_unrecalled_all(areas_evcats, filename=None):
    figsize = (5, 2.5)
    fig = plt.figure(figsize=figsize)
    ax = plt.subplot(1, 2, 1, aspect='equal')
    ax.plot(areas_evcats['recalled']['all']['conf'],
            areas_evcats['unrecalled']['all']['conf'], 'k.', alpha=0.4)
    ax.set_xlabel('recalled', size='large')
    ax.set_ylabel('unrecalled', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))
    ax.set_xticks(np.arange(0.1, 1, 0.2))
    ax.set_yticks(np.arange(0.1, 1, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.text(0.1, 0.85, 'A', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.01, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        areas_evcats['recalled']['all']['conf'],
        areas_evcats['unrecalled']['all']['conf'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='bottom',
            transform=ax.transAxes, size='large')
    ax.set_title("Confidence")

    ax = plt.subplot(1, 2, 2, aspect='equal')
    ax.plot(areas_evcats['recalled']['all']['rt'],
            areas_evcats['unrecalled']['all']['rt'], 'k.', alpha=0.4)
    ax.set_xlabel('recalled', size='large')
    ax.set_ylabel('unrecalled', size='large')
    ax.plot([0, 1], [0, 1], 'k-')
    ax.set_xlim((0, 1))
    ax.set_ylim((0, 1))
    ax.set_xticks(np.arange(0.1, 1, 0.2))
    ax.set_yticks(np.arange(0.1, 1, 0.2))
    ax.set_xticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.set_yticklabels(['.1', '.3', '.5', '.7', '.9'], size='large')
    ax.text(0.1, 0.85, 'B', horizontalalignment='center',
            verticalalignment='center',
            transform=ax.transAxes, size='large', weight='bold')
    ax.text(0.95, 0.01, r'$\mathsf{r=}$' + "{:.2f}".format(np.corrcoef(
        areas_evcats['recalled']['all']['rt'],
        areas_evcats['unrecalled']['all']['rt'])[0, 1])[-3:],
            horizontalalignment='right', verticalalignment='bottom',
            transform=ax.transAxes, size='large')
    ax.set_title("Latency")
    plt.subplots_adjust(left=0.12, right=0.99, top=0.90, bottom=0.23,
                        wspace=0.5, hspace=0)
    fig.set_size_inches(figsize)
    if filename is not None:
        fig.savefig(filename)

make_corr_aucs_fig_recalled_unrecalled_all(
    areas_evcats_subj, './figs/correlations_auc_recunrec_all_subj.pdf')



recalled_bars_subj = [
    areas_evcats_subj['recalled']['all']['conf'].mean(),
    areas_evcats_subj['recalled']['all']['rt'].mean(),
    areas_evcats_subj['recalled']['all']['old']['conf'].mean(),
    areas_evcats_subj['recalled']['all']['old']['rt'].mean(),
    areas_evcats_subj['recalled']['all']['new']['conf'].mean(),
    areas_evcats_subj['recalled']['all']['new']['rt'].mean()]
unrecalled_bars_subj = [
    areas_evcats_subj['unrecalled']['all']['conf'].mean(),
    areas_evcats_subj['unrecalled']['all']['rt'].mean(),
    areas_evcats_subj['unrecalled']['all']['old']['conf'].mean(),
    areas_evcats_subj['unrecalled']['all']['old']['rt'].mean(),
    areas_evcats_subj['unrecalled']['all']['new']['conf'].mean(),
    areas_evcats_subj['unrecalled']['all']['new']['rt'].mean()]
recalled_95CI_subj = np.array([
    areas_evcats_subj['recalled']['all']['conf'],
    areas_evcats_subj['recalled']['all']['rt'],
    areas_evcats_subj['recalled']['all']['old']['conf'],
    areas_evcats_subj['recalled']['all']['old']['rt'],
    areas_evcats_subj['recalled']['all']['new']['conf'],
    areas_evcats_subj['recalled']['all']['new']['rt']])
recalled_95CI_subj = ((np.std(recalled_95CI_subj, 1, ddof=1) /
                       np.sqrt(recalled_95CI_subj.shape[1] - 1)) *
                      dists.t.ppf(0.975, recalled_95CI_subj.shape[1] - 1))

unrecalled_95CI_subj = np.array([
    areas_evcats_subj['unrecalled']['all']['conf'],
    areas_evcats_subj['unrecalled']['all']['rt'],
    areas_evcats_subj['unrecalled']['all']['old']['conf'],
    areas_evcats_subj['unrecalled']['all']['old']['rt'],
    areas_evcats_subj['unrecalled']['all']['new']['conf'],
    areas_evcats_subj['unrecalled']['all']['new']['rt']])
unrecalled_95CI_subj = ((np.std(unrecalled_95CI_subj, 1, ddof=1) /
                         np.sqrt(unrecalled_95CI_subj.shape[1] - 1)) *
                        dists.t.ppf(0.975, unrecalled_95CI_subj.shape[1] - 1))

cp = {'recalled':
      (distplots_fullroc_subjmean['recalled']['all']['conf']['lure']['new'][
          'ROC'].mean(0)[-1],
       distplots_fullroc_subjmean['recalled']['all']['conf']['targ']['new'][
           'ROC'].mean(0)[-1]),
      'unrecalled':
      (distplots_fullroc_subjmean['unrecalled']['all']['conf']['lure']['new'][
          'ROC'].mean(0)[-1],
       distplots_fullroc_subjmean['unrecalled']['all']['conf']['targ']['new'][
           'ROC'].mean(0)[-1])}

rand_area = {}
for recunrec in cp:
    rand_area[recunrec] = (
        (cp[recunrec][0] * cp[recunrec][1]) / 2. +
        ((1. - cp[recunrec][0]) * (1. - cp[recunrec][1])) / 2 +
        cp[recunrec][1] * (1 - cp[recunrec][0]))

plt.figure(figsize=(4, 3))
ax = plt.subplot(111)
ax.bar(np.arange(-0.4, 11, 2), recalled_bars_subj, yerr=recalled_95CI_subj,
       color='grey', error_kw={'ecolor': 'black'}, label='recalled')
ax.bar(np.arange(0.4, 11, 2), unrecalled_bars_subj, yerr=unrecalled_95CI_subj,
       color='white', error_kw={'ecolor': 'black'}, label='unrecalled')
ax.bar(np.arange(-0.4, 11, 2), [rand_area['recalled'],
                                rand_area['recalled'], 0, 0, 0, 0],
       color='black', alpha=0.3)
ax.bar(np.arange(0.4, 11, 2), [rand_area['unrecalled'],
                               rand_area['unrecalled'], 0, 0, 0, 0],
       color='black', alpha=0.3)
ax.set_ylim((0.5, 1))
ax.set_xlim((-0.8, 11.6))
ax.set_xticks(np.arange(0.4, 11, 2))
ax.set_xticklabels(['C', 'L', 'C("O")', 'L("O")', 'C("N")', 'L("N")'])
ax.set_ylabel("AUC")
ax.set_xlabel("Dependent variable")
ax.legend(loc='upper right')
plt.subplots_adjust(left=0.14, right=0.99, top=0.97, bottom=0.15,
                    wspace=0.0, hspace=0.05)
plt.savefig('./figs/recalled_unrecalled_aucs_shading_subj.pdf')

for evcat in events:
    print evcat
    make_corr_aucs_fig(areas_evcats_subj[evcat]['all']['conf'],
                       areas_evcats_subj[evcat]['all']['rt'],
                       oldnew_areas_subj[evcat],
                       './figs/correlations_aucs_' +
                       evcat+'_subj.pdf')


for evcat in events:
    print evcat
    make_corr_aucs_fig_oldnew(oldnew_areas_subj[evcat],
                              './figs/correlations_aucs_oldnew_' +
                              evcat + '_subj.pdf')








###################################################################################################
###################################################################################################
###################################################################################################
###
### Code snippets to calculate statistics etc.:
###
###



# targfilt = data_targlurefilt['type'] == 'RECOG_TARGET'
# lurefilt = data_targlurefilt['type'] == 'RECOG_LURE'
# oldfilt = data_targlurefilt['recog_resp'] == 'old'
# newfilt = data_targlurefilt['recog_resp'] == 'new'
# hitfilt = targfilt & oldfilt
# fafilt = lurefilt & oldfilt
# missfilt = targfilt & newfilt
# crfilt = lurefilt & newfilt

# plt.figure()
# plt.hist(data_targlurefilt[
#     hitfilt]['recog_rt'], 100, alpha=0.2)
# plt.hist(data_targlurefilt[
#     fafilt]['recog_rt'], 100, alpha=0.2)
# plt.hist(data_targlurefilt[
#     crfilt]['recog_rt'], 100, alpha=0.2)
# plt.hist(data_targlurefilt[
#     missfilt]['recog_rt'], 100, alpha=0.2)
# data_targlurefilt[hitfilt]['recog_rt'].mean()
# data_targlurefilt[hitfilt]['recog_rt'].std()
# data_targlurefilt[fafilt]['recog_rt'].mean()
# data_targlurefilt[fafilt]['recog_rt'].std()
# data_targlurefilt[crfilt]['recog_rt'].mean()
# data_targlurefilt[crfilt]['recog_rt'].std()
# data_targlurefilt[missfilt]['recog_rt'].mean()
# data_targlurefilt[missfilt]['recog_rt'].std()



# # Confidence vs. Latency comparisons (subj based):
# SE_areassubjCvL = {}
# t_areassubjCvL = {}
# df_areassubjCvL = {}
# p_areassubjCvL = {}
# for evcat in events:
#     SE_areassubjCvL[evcat] = (np.std(
#         areas_evcats_subj[evcat]['all']['conf']-
#         areas_evcats_subj[evcat]['all']['rt'],ddof=1)/np.sqrt(len(
#             areas_evcats_subj[evcat]['all']['conf'])-1))
#     t_areassubjCvL[evcat] = (
#         np.mean(areas_evcats_subj[evcat]['all']['conf']-
#                 areas_evcats_subj[evcat]['all']['rt'])/SE_areassubjCvL[evcat])
#     df_areassubjCvL[evcat] = len(
#         areas_evcats_subj[evcat]['all']['conf'])-1
#     p_areassubjCvL[evcat] =  scipy.stats.t.sf(
#         np.float(t_areassubjCvL[evcat]),
#         df_areassubjCvL[evcat])

# # In [63]: t_areassubjCvL
# # Out[63]: 
# # {'all_recunrec': 13.742099094042926928,
# #  'all_targlure': 13.608516982518349249,
# #  'recalled': 12.098457854224814002,
# #  'unrecalled': 16.568839805680454292}
# # Cohen's d
# # In [156]: 13.608516982518349256/np.sqrt(171)
# # Out[156]: 1.04066930347815

# # In [64]: df_areassubjCvL
# # Out[64]: {'all_recunrec': 170, 'all_targlure': 170, 'recalled': 170, 'unrecalled': 170}

# # In [65]: SE_areassubjCvL
# # Out[65]: 
# # {'all_recunrec': 0.0027499061344639383221,
# #  'all_targlure': 0.002790863351393547646,
# #  'recalled': 0.0032669155155805191534,
# #  'unrecalled': 0.0022922835821003887115}

# # In [66]: p_areassubjCvL
# # Out[66]: 
# # {'all_recunrec': 1.1047026802772388e-29,
# #  'all_targlure': 2.6484724577765631e-29,
# #  'recalled': 5.2574658691967222e-25,
# #  'unrecalled': 1.2735476575045244e-37}



# # AUCs larger than 0.5 comparisons (subj based):
# SE_areassubj05 = {}
# t_areassubj05 = {}
# df_areassubj05 = {}
# p_areassubj05 = {}
# for evcat in events:
#     SE_areassubj05[evcat] = {}
#     t_areassubj05[evcat] = {}
#     df_areassubj05[evcat] = {}
#     p_areassubj05[evcat] = {}
#     for dv in ['conf','rt']: #SE_areassubj05:
#         SE_areassubj05[evcat][dv] = (
#             np.std(areas_evcats_subj[evcat]['all'][dv]-0.5,ddof=1)/
#             np.sqrt(len(areas_evcats_subj[evcat]['all'][dv])-1))
#         t_areassubj05[evcat][dv] = (
#             np.mean(areas_evcats_subj[evcat]['all'][dv]-0.5)/
#             SE_areassubj05[evcat][dv])
#         df_areassubj05[evcat][dv] = len(
#             areas_evcats_subj[evcat]['all'][dv])-1
#         p_areassubj05[evcat][dv] = scipy.stats.t.sf(
#             np.float(t_areassubj05[evcat][dv]),
#             df_areassubj05[evcat][dv])

# # In [74]: t_areassubj05
# # Out[74]: 
# # {'all_recunrec': {'conf': 109.9167437013066469, 'rt': 65.966760065435380088},
# #  'all_targlure': {'conf': 109.31210819412167484, 'rt': 65.243003508956946293},
# #  'recalled': {'conf': 164.96146564567252746, 'rt': 78.710996041080799605},
# #  'unrecalled': {'conf': 85.358162961732308861, 'rt': 57.37502136925626638}}
# # Cohen's d
# # In [157]: 109.31210819412167484/np.sqrt(171)
# # Out[157]: 8.3593058407642236
# # In [158]: 65.243003508956946293/np.sqrt(171)
# # Out[158]: 4.9892571766423313


# # In [75]: df_areassubj05
# # Out[75]: 
# # {'all_recunrec': {'conf': 170, 'rt': 170},
# #  'all_targlure': {'conf': 170, 'rt': 170},
# #  'recalled': {'conf': 170, 'rt': 170},
# #  'unrecalled': {'conf': 170, 'rt': 170}}

# # In [76]: SE_areassubj05
# # Out[76]: 
# # {'all_recunrec': {'conf': 0.0039087808938148929476,
# #   'rt': 0.0059401277962143571362},
# #  'all_targlure': {'conf': 0.0039439430272850970153,
# #   'rt': 0.0060257988512552211437},
# #  'recalled': {'conf': 0.0027838204966708408761,
# #   'rt': 0.0053321453247481853289},
# #  'unrecalled': {'conf': 0.0045603614555820722428,
# #   'rt': 0.0061225876425938166561}}

# # In [77]: p_areassubj05
# # Out[77]: 
# # {'all_recunrec': {'conf': 3.7978892063369358e-160,
# #   'rt': 2.4070585026165763e-123},
# #  'all_targlure': {'conf': 9.5750897824612098e-160,
# #   'rt': 1.4629230077246216e-122},
# #  'recalled': {'conf': 7.755651438418654e-190, 'rt': 5.671637691203939e-136},
# #  'unrecalled': {'conf': 8.2238302293289177e-142,
# #   'rt': 1.7472242604522207e-113}}


# # AUCs larger than 0.5 comparisons (subj based):
# SE_oldnewareassubj05 = {}
# t_oldnewareassubj05 = {}
# df_oldnewareassubj05 = {}
# p_oldnewareassubj05 = {}
# for evcat in events:
#     SE_oldnewareassubj05[evcat] = {}
#     t_oldnewareassubj05[evcat] = {}
#     df_oldnewareassubj05[evcat] = {}
#     p_oldnewareassubj05[evcat] = {}
#     for dv in ['conf', 'rt']:
#         SE_oldnewareassubj05[evcat][dv] = {}
#         t_oldnewareassubj05[evcat][dv] = {}
#         df_oldnewareassubj05[evcat][dv] = {}
#         p_oldnewareassubj05[evcat][dv] = {}
#         for oldnew in ['old', 'new']:
#             SE_oldnewareassubj05[evcat][dv][oldnew] = (
#                 np.std(oldnew_areas_subj[evcat][dv][oldnew] - 0.5, ddof=1) /
#                 np.sqrt(len(oldnew_areas_subj[evcat][dv][oldnew]) - 1))
#             t_oldnewareassubj05[evcat][dv][oldnew] = (
#                 np.mean(oldnew_areas_subj[evcat][dv][oldnew] - 0.5) /
#                 SE_oldnewareassubj05[evcat][dv][oldnew])
#             df_oldnewareassubj05[evcat][dv][oldnew] = len(
#                 oldnew_areas_subj[evcat][dv][oldnew]) - 1
#             p_oldnewareassubj05[evcat][dv][oldnew] = scipy.stats.t.sf(
#                 np.float(t_oldnewareassubj05[evcat][dv][oldnew]),
#                 df_oldnewareassubj05[evcat][dv][oldnew])



# # In [33]: t_oldnewareassubj05
# # Out[33]: 
# # {'all_recunrec': {'conf': {'new': 36.711804885411639422,
# #    'old': 59.612675031174615099},
# #   'rt': {'new': 19.010419509901910623, 'old': 19.067425394918745536}},
# #  'all_targlure': {'conf': {'new': 36.580201042088173143,
# #    'old': 59.920775696433953433},
# #   'rt': {'new': 19.085441507413500772, 'old': 18.932539335953322501}},
# #  'recalled': {'conf': {'new': 28.179856550515845003,
# #    'old': 63.321808691743646624},
# #   'rt': {'new': 13.26316190277367104, 'old': 20.637965088742933131}},
# #  'unrecalled': {'conf': {'new': 34.332745155747945626,
# #    'old': 54.611872591268136701},
# #   'rt': {'new': 19.212554010681483651, 'old': 15.338558467644890323}}}
# # Cohen's d:
# # In [1]: 18.932539335953322501/np.sqrt(171)
# # Out[1]: 1.447807468597003
# # In [2]: 59.920775696433953433/np.sqrt(171)
# # Out[2]: 4.5822562434969054
# # Cohen's d:
# # In [6]: 13.26316190277367104/np.sqrt(171)
# # Out[6]: 1.0142593404562994
# # In [7]: 63.321808691743646624/np.sqrt(171)
# # Out[7]: 4.8423397370092287


# # Old vs. New comparisons (subj based):
# SE_oldnewareassubjOvN = {}
# t_oldnewareassubjOvN = {}
# df_oldnewareassubjOvN = {}
# p_oldnewareassubjOvN = {}
# for evcat in events:
#     SE_oldnewareassubjOvN[evcat] = {}
#     t_oldnewareassubjOvN[evcat] = {}
#     df_oldnewareassubjOvN[evcat] = {}
#     p_oldnewareassubjOvN[evcat] = {}
#     for dv in ['conf', 'rt']:
#         SE_oldnewareassubjOvN[evcat][dv] = (
#             np.std(oldnew_areas_subj[evcat][dv]['old'] -
#                    oldnew_areas_subj[evcat][dv]['new'], ddof=1) /
#             np.sqrt(len(oldnew_areas_subj[evcat][dv]['old']) - 1))
#         t_oldnewareassubjOvN[evcat][dv] = (np.mean(
#             oldnew_areas_subj[evcat][dv]['old'] -
#             oldnew_areas_subj[evcat][dv]['new']) /
#             SE_oldnewareassubjOvN[evcat][dv])
#         df_oldnewareassubjOvN[evcat][dv] = len(
#             oldnew_areas_subj[evcat][dv]['old']) - 1
#         p_oldnewareassubjOvN[evcat][dv] =  scipy.stats.t.sf(
#             np.float(t_oldnewareassubjOvN[evcat][dv]),
#             df_oldnewareassubjOvN[evcat][dv])



# # In [43]: t_oldnewareassubjOvN
# # Out[43]: 
# # {'all_recunrec': {'conf': 37.135181146033168883, 'rt': 11.441285592523902582},
# #  'all_targlure': {'conf': 36.076078450590285476, 'rt': 11.336865535650178942},
# #  'recalled': {'conf': 26.386980560839320632, 'rt': 10.583919709157453286},
# #  'unrecalled': {'conf': 30.426383787087853955, 'rt': 6.0368965258211625849}}
# # Cohen's d:
# # In [3]: 36.076078450590285476/np.sqrt(171)
# # Out[3]: 2.7588066709713766
# # In [4]: 11.336865535650178942/np.sqrt(171)
# # Out[4]: 0.86695177555102176



# # In [44]: df_oldnewareassubjOvN
# # Out[44]: 
# # {'all_recunrec': {'conf': 170, 'rt': 170},
# #  'all_targlure': {'conf': 170, 'rt': 170},
# #  'recalled': {'conf': 170, 'rt': 170},
# #  'unrecalled': {'conf': 170, 'rt': 170}}

# # In [45]: SE_oldnewareassubjOvN
# # Out[45]: 
# # {'all_recunrec': {'conf': 0.0048157450060603189931,
# #   'rt': 0.0066855739736330428958},
# #  'all_targlure': {'conf': 0.0049482927250001060104,
# #   'rt': 0.0064433154067407208406},
# #  'recalled': {'conf': 0.0071779483890045407817,
# #   'rt': 0.0079699119942388560322},
# #  'unrecalled': {'conf': 0.0046162775588690277655,
# #   'rt': 0.0066920351390245130216}}

# # In [46]: p_oldnewareassubjOvN
# # Out[46]: 
# # {'all_recunrec': {'conf': 8.7750701283211094e-84,
# #   'rt': 3.8412057123362241e-23},
# #  'all_targlure': {'conf': 6.9235257043030977e-82,
# #   'rt': 7.5791903582469874e-23},
# #  'recalled': {'conf': 2.6265357230491876e-62, 'rt': 9.913759135573389e-21},
# #  'unrecalled': {'conf': 5.412224254698999e-71, 'rt': 4.7918270457993108e-09}}



# # Confidence vs. Latency comparisons (subj. based):
# SE_oldnewareassubjCvL = {}
# t_oldnewareassubjCvL = {}
# df_oldnewareassubjCvL = {}
# p_oldnewareassubjCvL = {}
# for evcat in events:
#     SE_oldnewareassubjCvL[evcat] = {}
#     t_oldnewareassubjCvL[evcat] = {}
#     df_oldnewareassubjCvL[evcat] = {}
#     p_oldnewareassubjCvL[evcat] = {}
#     for resp in ['old', 'new']:
#         SE_oldnewareassubjCvL[evcat][resp] = (np.std(
#             oldnew_areas_subj[evcat]['conf'][resp] -
#             oldnew_areas_subj[evcat]['rt'][resp], ddof=1) / np.sqrt(len(
#                 oldnew_areas_subj[evcat]['conf'][resp]) - 1))
#         t_oldnewareassubjCvL[evcat][resp] = (np.mean(
#             oldnew_areas_subj[evcat]['conf'][resp] -
#             oldnew_areas_subj[evcat]['rt'][resp]) /
#             SE_oldnewareassubjCvL[evcat][resp])
#         df_oldnewareassubjCvL[evcat][resp] = len(
#             oldnew_areas_subj[evcat]['conf'][resp]) - 1
#         p_oldnewareassubjCvL[evcat][resp] =  scipy.stats.t.sf(
#             np.float(t_oldnewareassubjCvL[evcat][resp]),
#             df_oldnewareassubjCvL[evcat][resp])

# # In [53]: t_oldnewareassubjCvL
# # Out[53]: 
# # {'all_recunrec': {'new': 17.133141204160245229, 'old': 21.976510836238392903},
# #  'all_targlure': {'new': 17.781494455134877242, 'old': 23.038229058815140403},
# #  'recalled': {'new': 10.852167034343894601, 'old': 20.992115053055392208},
# #  'unrecalled': {'new': 17.460411562753113216, 'old': 23.149143871277820717}}

# # In [54]: df_oldnewareassubjCvL
# # Out[54]: 
# # {'all_recunrec': {'new': 170, 'old': 170},
# #  'all_targlure': {'new': 170, 'old': 170},
# #  'recalled': {'new': 170, 'old': 170},
# #  'unrecalled': {'new': 170, 'old': 170}}

# # In [55]: SE_oldnewareassubjCvL
# # Out[55]: 
# # {'all_recunrec': {'new': 0.004477790659987935183,
# #   'old': 0.0081478185033746748469},
# #  'all_targlure': {'new': 0.0043685694259229706692,
# #   'old': 0.0079497295009066414472},
# #  'recalled': {'new': 0.0072910141113910875918,
# #   'old': 0.0087735217957974977248},
# #  'unrecalled': {'new': 0.0042666079739232599806,
# #   'old': 0.0075404188234633351815}}

# # In [56]: p_oldnewareassubjCvL
# # Out[56]: 
# # {'all_recunrec': {'new': 3.5742893297177424e-39,
# #   'old': 7.4482008654786336e-52},
# #  'all_targlure': {'new': 6.1294153280059279e-41,
# #   'old': 1.8169585590507418e-54},
# #  'recalled': {'new': 1.7572372138713962e-21, 'old': 2.2366102160142831e-49},
# #  'unrecalled': {'new': 4.5656242657770748e-40, 'old': 9.771585383536432e-55}}





# # calculate individual classification points:

# tmp = (distplots_fullroc['recalled']['all']['conf']['lure']['new']['ROC'][:,-1],
#        distplots_fullroc['recalled']['all']['conf']['targ']['new']['ROC'][:,-1])

# def get_randROCareas(dat,selector=None):
#     if selector is None:
#         classification_points = (
#             dat['lure']['new']['ROC'][:,-1],
#             dat['targ']['new']['ROC'][:,-1])
#     else:
#         classification_points = (
#             dat['lure']['new']['ROC'][selector][:,-1],
#             dat['targ']['new']['ROC'][selector][:,-1])
#     # return classification_points
#     # calculate areas under the random ROCs:
#     # base of the triangle spanned by the random ROCs is the main diagonal:
#     base = np.sqrt(2)
#     # the left sides of the triangles spanned by random ROCs and main diagonal:
#     left_sides = np.sqrt(classification_points[0]**2+classification_points[1]**2)
#     # the angles in the left corners of that triangles is calculated by
#     # getting the angles spanned between the left legs of the random ROCs
#     # and the x-axis and subtracting 45 degree (angle of the main
#     # diagonal) from it:
#     left_angles = np.arcsin(classification_points[1]/left_sides)-np.deg2rad(45)
#     # the heights of the triangles (for area calculation):
#     heights = np.sin(left_angles)*left_sides
#     # the areas in the triangle formed by the random ROCs and the main
#     # diagonal plus the areas under the main diagonal:
#     return(((base*heights)/2)+0.5)

# randROCareas = {}
# randROCareas_subj = {}
# for evcat in events:
#     randROCareas[evcat] = get_randROCareas(
#         distplots_fullroc[evcat]['all']['conf'])
#     subj_array = np.array(resprates_dict_agg[evcat]['indiv_resprates']['conf']['subject'])
#     randROCareas_subj[evcat] = np.array(
#         [np.mean(randROCareas[evcat][subj_array==subj])
#          for subj in np.unique(subj_array)])
                                             
# # comparing recalled vs. unrecalled classification performance
# diff_recunrec = randROCareas['recalled']-randROCareas['unrecalled']
# df_recunrec = len(randROCareas['recalled'])-1
# SE_recunrec = np.std(diff_recunrec,ddof=1)/np.sqrt(df_recunrec)
# t_recunrec = np.mean(diff_recunrec)/SE_recunrec
# p_recunrec = scipy.stats.t.sf(np.float(t_recunrec),df_recunrec)
# # In [265]: t_recunrec
# # Out[265]: 86.574954527300904336
# # In [266]: df_recunrec
# # Out[266]: 2579
# # In [267]: SE_recunrec
# # Out[267]: 0.00081159208673057190393
# # In [268]: p_recunrec
# # Out[268]: 0.0

# # comparing recalled vs. unrecalled classification performance (subj. based)
# diff_recunrec_subj = randROCareas_subj['recalled']-randROCareas_subj['unrecalled']
# df_recunrec_subj = len(randROCareas_subj['recalled'])-1
# SE_recunrec_subj = np.std(diff_recunrec_subj,ddof=1)/np.sqrt(df_recunrec_subj)
# t_recunrec_subj = np.mean(diff_recunrec_subj)/SE_recunrec_subj
# p_recunrec_subj = scipy.stats.t.sf(np.float(t_recunrec_subj),df_recunrec_subj)
# # In [270]: t_recunrec_subj
# # Out[270]: 36.264897090999160076
# # In [271]: df_recunrec_subj
# # Out[271]: 170
# # In [272]: SE_recunrec_subj
# # Out[272]: 0.0018623038774995901456
# # In [273]: p_recunrec_subj
# # Out[273]: 3.1548585768968953e-82
# # Cohen's d
# # In [5]: 36.264897090999160076/np.sqrt(171)
# # Out[5]: 2.7732459932906579



# recalled_prop = []
# for subj in np.unique(events['all_recunrec']['participant']):
#     for sess in np.unique(
#             events['all_recunrec'][
#                 events['all_recunrec']['participant'] == subj]['session']):
#         targs = np.sum(events['all_recunrec'][
#             (events['all_recunrec']['participant'] == subj) &
#             (events['all_recunrec']['session'] == sess)][
#                 'type'] == 'RECOG_TARGET')
#         recalled = np.sum(events['all_recunrec'][
#             (events['all_recunrec']['participant'] == subj) &
#             (events['all_recunrec']['session'] == sess)][
#                 'recalled'] == 1)
#         recalled_prop.append((subj, sess, targs, recalled,
#                               np.float(recalled) / targs))

# # recprop = np.array(np.array(recalled_prop)[:, 4], np.float)
# # In [374]: recprop.mean()
# # Out[374]: 0.6068391989171491
# # In [376]: np.std(recprop)
# # Out[376]: 0.15767675085283403




# evcat = 'all_targlure'
# conf_areas_subj=areas_evcats_subj[evcat]['all']['conf']
# rt_areas_subj=areas_evcats_subj[evcat]['all']['rt'],
# SE_r_aucCL_subj = np.sqrt((1-np.corrcoef(conf_areas_subj,rt_areas_subj)[0,1])/(len(conf_areas_subj)-2))
# t_r_aucCL_subj = np.corrcoef(conf_areas_subj,rt_areas_subj)[0,1]/SE_r_aucCL_subj
# df_r_aucCL_subj = len(conf_areas_subj)-2
# p_aucCL_subj = scipy.stats.t.sf(np.float(t_r_aucCL_subj),df_r_aucCL_subj)
# In [360]: np.corrcoef(conf_areas_subj,rt_areas_subj)[0,1]
# Out[360]: 0.92731463590019259
# In [105]: t_r_aucCL_subj
# Out[105]: 44.714356136752208
# In [106]: df_r_aucCL_subj
# Out[106]: 169
# In [107]: SE_r_aucCL_subj
# Out[107]: 0.020738633316426131
# In [108]: p_aucCL_subj
# Out[108]: 7.1915636725523029e-96




# evcat = 'all_recunrec'
# oldnew_areas_stats=oldnew_areas[evcat]
# SE_r_old = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats['conf']['old'],oldnew_areas_stats['rt']['old'])[0,1])/
#                    (len(oldnew_areas_stats['conf']['old'])-2))
# t_r_old = np.corrcoef(oldnew_areas_stats['conf']['old'],
#                       oldnew_areas_stats['rt']['old'])[0,1]/SE_r_old
# df_r_old = len(oldnew_areas_stats['conf']['old'])-2
# p_old = scipy.stats.t.sf(np.float(t_r_old),df_r_old)
# In [110]: t_r_old
# Out[110]: 18.882290305315273
# In [111]: df_r_old
# Out[111]: 2578
# In [112]: p_old
# Out[112]: 6.8570203368680093e-75

# evcat = 'all_targlure'
# oldnew_areas_stats=oldnew_areas[evcat]
# SE_r_old = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats['conf']['old'],oldnew_areas_stats['rt']['old'])[0,1])/
#                    (len(oldnew_areas_stats['conf']['old'])-2))
# t_r_old = np.corrcoef(oldnew_areas_stats['conf']['old'],
#                       oldnew_areas_stats['rt']['old'])[0,1]/SE_r_old
# df_r_old = len(oldnew_areas_stats['conf']['old'])-2
# p_old = scipy.stats.t.sf(np.float(t_r_old),df_r_old)
# In [334]: np.corrcoef(oldnew_areas_stats['conf']['old'],oldnew_areas_stats['rt']['old'])[0,1]
# Out[334]: 0.30957495567955157
# In [114]: t_r_old
# Out[114]: 20.803940672136758
# In [115]: df_r_old
# Out[115]: 3118
# In [116]: p_old
# Out[116]: 2.0114011249871564e-90


# # subj. based analyses

# evcat = 'all_recunrec'
# oldnew_areas_stats_subj=oldnew_areas_subj[evcat]
# SE_r_old_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats_subj['conf']['old'],oldnew_areas_stats_subj['rt']['old'])[0,1])/
#     (len(oldnew_areas_stats_subj['conf']['old'])-2))
# t_r_old_subj = np.corrcoef(oldnew_areas_stats_subj['conf']['old'],
#                       oldnew_areas_stats_subj['rt']['old'])[0,1]/SE_r_old_subj
# df_r_old_subj = len(oldnew_areas_stats_subj['conf']['old'])-2
# p_old_subj = scipy.stats.t.sf(np.float(t_r_old_subj),df_r_old_subj)
# In [118]: t_r_old_subj
# Out[118]: 7.398766996454726
# In [119]: df_r_old_subj
# Out[119]: 169
# In [120]: p_old_subj
# Out[120]: 3.0712955688482367e-12


# evcat = 'all_targlure'
# oldnew_areas_stats_subj=oldnew_areas_subj[evcat]
# SE_r_old_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats_subj['conf']['old'],oldnew_areas_stats_subj['rt']['old'])[0,1])/
#     (len(oldnew_areas_stats_subj['conf']['old'])-2))
# t_r_old_subj = np.corrcoef(oldnew_areas_stats_subj['conf']['old'],
#                       oldnew_areas_stats_subj['rt']['old'])[0,1]/SE_r_old_subj
# df_r_old_subj = len(oldnew_areas_stats_subj['conf']['old'])-2
# p_old_subj = scipy.stats.t.sf(np.float(t_r_old_subj),df_r_old_subj)
# In [361]: np.corrcoef(oldnew_areas_stats_subj['conf']['old'],oldnew_areas_stats_subj['rt']['old'])[0,1]
# Out[361]: 0.44886037275688934
# In [122]: t_r_old_subj
# Out[122]: 7.860022676769697
# In [123]: df_r_old_subj
# Out[123]: 169
# In [124]: p_old_subj
# Out[124]: 2.1478066108095257e-13


# evcat = 'all_recunrec'
# oldnew_areas_stats=oldnew_areas[evcat]
# SE_r_new = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats['conf']['new'],oldnew_areas_stats['rt']['new'])[0,1])/
#     (len(oldnew_areas_stats['conf']['new'])-2))
# t_r_new = np.corrcoef(oldnew_areas_stats['conf']['new'],
#                       oldnew_areas_stats['rt']['new'])[0,1]/SE_r_new
# df_r_new = len(oldnew_areas_stats['conf']['new'])-2
# p_new = scipy.stats.t.sf(np.float(t_r_new),df_r_new)
# In [126]: t_r_new
# Out[126]: 25.777282254839673
# In [127]: df_r_new
# Out[127]: 2578
# In [128]: p_new
# Out[128]: 7.3051207425639834e-131

# evcat = 'all_targlure'
# oldnew_areas_stats=oldnew_areas[evcat]
# SE_r_new = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats['conf']['new'],oldnew_areas_stats['rt']['new'])[0,1])/
#     (len(oldnew_areas_stats['conf']['new'])-2))
# t_r_new = np.corrcoef(oldnew_areas_stats['conf']['new'],
#                       oldnew_areas_stats['rt']['new'])[0,1]/SE_r_new
# df_r_new = len(oldnew_areas_stats['conf']['new'])-2
# p_new = scipy.stats.t.sf(np.float(t_r_new),df_r_new)
# In [335]: np.corrcoef(oldnew_areas_stats['conf']['new'],oldnew_areas_stats['rt']['new'])[0,1]
# Out[335]: 0.39054636548778676
# In [130]: t_r_new
# Out[130]: 27.934462562105114
# In [131]: df_r_new
# Out[131]: 3118
# In [132]: p_new
# Out[132]: 9.4421753066921573e-154

# # subj. based analyses

# evcat = 'all_recunrec'
# oldnew_areas_stats_subj=oldnew_areas_subj[evcat]
# SE_r_new_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats_subj['conf']['new'],oldnew_areas_stats_subj['rt']['new'])[0,1])/
#     (len(oldnew_areas_stats_subj['conf']['new'])-2))
# t_r_new_subj = np.corrcoef(oldnew_areas_stats_subj['conf']['new'],
#                       oldnew_areas_stats_subj['rt']['new'])[0,1]/SE_r_new_subj
# df_r_new_subj = len(oldnew_areas_stats_subj['conf']['new'])-2
# p_new_subj = scipy.stats.t.sf(np.float(t_r_new_subj),df_r_new_subj)
# In [134]: t_r_new_subj
# Out[134]: 10.202698409965297
# In [135]: df_r_new_subj
# Out[135]: 169
# In [136]: p_new_subj
# Out[136]: 1.2012754508918692e-19


# evcat = 'all_targlure'
# oldnew_areas_stats_subj=oldnew_areas_subj[evcat]
# SE_r_new_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_stats_subj['conf']['new'],oldnew_areas_stats_subj['rt']['new'])[0,1])/
#     (len(oldnew_areas_stats_subj['conf']['new'])-2))
# t_r_new_subj = np.corrcoef(oldnew_areas_stats_subj['conf']['new'],
#                       oldnew_areas_stats_subj['rt']['new'])[0,1]/SE_r_new_subj
# df_r_new_subj = len(oldnew_areas_stats_subj['conf']['new'])-2
# p_new_subj = scipy.stats.t.sf(np.float(t_r_new_subj),df_r_new_subj)
# In [364]: np.corrcoef(oldnew_areas_stats_subj['conf']['new'],oldnew_areas_stats_subj['rt']['new'])[0,1]
# Out[364]: 0.56243787106949494
# In [138]: t_r_new_subj
# Out[138]: 11.053454926948879
# In [139]: df_r_new_subj
# Out[139]: 169
# In [140]: p_new_subj
# Out[140]: 5.086040647368219e-22





# # subj. based analysis
# r_C_subj = np.corrcoef(
#     oldnew_areas_subj['recalled']['conf']['old'],
#     oldnew_areas_subj['unrecalled']['conf']['old'])[0,1]
# df_r_C_subj = len(oldnew_areas_subj['recalled']['conf']['old'])-2
# SE_r_C_subj = np.sqrt((1-r_C_subj)/df_r_C_subj)
# t_r_C_subj = r_C_subj/SE_r_C_subj
# p_C_subj = scipy.stats.t.sf(np.float(t_r_C_subj),df_r_C_subj)
# In [184]: r_C_subj
# Out[184]: 0.89262200213732457
# In [185]: t_r_C_subj
# Out[185]: 35.412231024000178
# In [186]: df_r_C_subj
# Out[186]: 169
# In [187]: p_C_subj
# Out[187]: 2.1055959853560539e-80


# # # subj. based analysis:
# r_C_subj = np.corrcoef(
#     oldnew_areas_subj['recalled']['conf']['new'],
#     oldnew_areas_subj['unrecalled']['conf']['new'])[0,1]
# df_r_C_subj = len(oldnew_areas_subj['recalled']['conf']['new'])-2
# SE_r_C_subj = np.sqrt((1-r_C_subj)/df_r_C_subj)
# t_r_C_subj = r_C_subj/SE_r_C_subj
# p_C_subj = scipy.stats.t.sf(np.float(t_r_C_subj),df_r_C_subj)
# In [194]: r_C_subj
# Out[194]: 0.56566754583178713
# In [195]: t_r_C_subj
# Out[195]: 11.158182849663696
# In [196]: df_r_C_subj
# Out[196]: 169
# In [197]: p_C_subj
# Out[197]: 2.5826592580892187e-22


# # subj. based analysis
# r_L_subj = np.corrcoef(
#     oldnew_areas_subj['recalled']['rt']['old'],
#     oldnew_areas_subj['unrecalled']['rt']['old'])[0,1]
# df_r_L_subj = len(oldnew_areas_subj['recalled']['rt']['old'])-2
# SE_r_L_subj = np.sqrt((1-r_L_subj)/df_r_L_subj)
# t_r_L_subj = r_L_subj/SE_r_L_subj
# p_L_subj = scipy.stats.t.sf(np.float(t_r_L_subj),df_r_L_subj)
# In [204]: r_L_subj
# Out[204]: 0.96957727449248632
# In [205]: t_r_L_subj
# Out[205]: 72.264785931434844
# In [206]: df_r_L_subj
# Out[206]: 169
# In [207]: p_L_subj
# Out[207]: 2.6434404287000102e-129
# In [214]: SE_r_L_subj
# Out[214]: 0.013417008879157625


# # subj. based analysis
# r_L_subj = np.corrcoef(
#     oldnew_areas_subj['recalled']['rt']['new'],
#     oldnew_areas_subj['unrecalled']['rt']['new'])[0,1]
# df_r_L_subj = len(oldnew_areas_subj['recalled']['rt']['new'])-2
# SE_r_L_subj = np.sqrt((1-r_L_subj)/df_r_L_subj)
# t_r_L_subj = r_L_subj/SE_r_L_subj
# p_L_subj = scipy.stats.t.sf(np.float(t_r_L_subj),df_r_L_subj)
# In [216]: r_L_subj
# Out[216]: 0.61775114303479517
# In [217]: df_r_L_subj
# Out[217]: 169
# In [218]: SE_r_L_subj
# Out[218]: 0.0475586748764842
# In [219]: t_r_L_subj
# Out[219]: 12.989242123317602
# In [220]: p_L_subj
# Out[220]: 1.689837083857559e-27

# # # subj. based analyis
# # SE_r_C_subj = np.sqrt((1-np.corrcoef(areas_evcats_subj['recalled']['all']['conf'],
# #                                      areas_evcats_subj['unrecalled']['all']['conf'])[0,1])/
# #                       (len(areas_evcats_subj['recalled']['all']['conf'])-2))
# # t_r_C_subj = np.corrcoef(areas_evcats_subj['recalled']['all']['conf'],
# #                          areas_evcats_subj['unrecalled']['all']['conf'])[0,1]/SE_r_C_subj
# # df_r_C_subj = len(areas_evcats_subj['recalled']['all']['conf'])-2
# # p_C_subj = scipy.stats.t.sf(np.float(t_r_C_subj),df_r_C_subj)
# # In [231]: SE_r_C_subj
# # Out[231]: 0.025995122383697031
# # In [232]: t_r_C_subj
# # Out[232]: 34.075579541295056
# # In [233]: df_r_C_subj
# # Out[233]: 169
# # In [234]: p_C_subj
# # Out[234]: 6.3448376659183168e-78
# # In [235]: np.corrcoef(areas_evcats_subj['recalled']['all']['conf'],areas_evcats_subj['unrecalled']['all']['conf'])[0,1]
# # Out[235]: 0.88579886047136769

# # subj. based analysis
# SE_r_L_subj = np.sqrt((1-np.corrcoef(areas_evcats_subj['recalled']['all']['rt'],
#                                      areas_evcats_subj['unrecalled']['all']['rt'])[0,1])/
#                       (len(areas_evcats_subj['recalled']['all']['rt'])-2))
# t_r_L_subj = np.corrcoef(areas_evcats_subj['recalled']['all']['rt'],
#                          areas_evcats_subj['unrecalled']['all']['rt'])[0,1]/SE_r_L_subj
# df_r_L_subj = len(areas_evcats_subj['recalled']['all']['rt'])-2
# p_L_subj = scipy.stats.t.sf(np.float(t_r_L_subj),df_r_L_subj)
# In [243]: SE_r_L_subj
# Out[243]: 0.018689425054202069
# In [244]: t_r_L_subj
# Out[244]: 50.347681021448224
# In [245]: df_r_L_subj
# Out[245]: 169
# In [246]: p_L_subj
# Out[246]: 5.6694148226299723e-104
# In [247]: np.corrcoef(areas_evcats_subj['recalled']['all']['rt'],areas_evcats_subj['unrecalled']['all']['rt'])[0,1]
# Out[247]: 0.94096921110322851
        
# # subj. based analyses

# evcat = 'all_recunrec'
# oldnew_areas_subj_stats=oldnew_areas_subj[evcat]
# SE_r_C_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_subj_stats['conf']['old'],oldnew_areas_subj_stats['conf']['new'])[0,1])/
#                       (len(oldnew_areas_subj_stats['conf']['old'])-2))
# t_r_C_subj = np.corrcoef(oldnew_areas_subj_stats['conf']['old'],
#                          oldnew_areas_subj_stats['conf']['new'])[0,1]/SE_r_C_subj
# df_r_C_subj = len(oldnew_areas_subj_stats['conf']['old'])-2
# p_C_subj = scipy.stats.t.sf(np.float(t_r_C_subj),df_r_C_subj)
# In [153]: t_r_C_subj
# Out[153]: 11.96619234959212
# In [154]: df_r_C_subj
# Out[154]: 169
# In [155]: p_C_subj
# Out[155]: 1.3510762033251969e-24

# evcat = 'all_targlure'
# oldnew_areas_subj_stats=oldnew_areas_subj[evcat]
# SE_r_C_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_subj_stats['conf']['old'],oldnew_areas_subj_stats['conf']['new'])[0,1])/
#                       (len(oldnew_areas_subj_stats['conf']['old'])-2))
# t_r_C_subj = np.corrcoef(oldnew_areas_subj_stats['conf']['old'],
#                          oldnew_areas_subj_stats['conf']['new'])[0,1]/SE_r_C_subj
# df_r_C_subj = len(oldnew_areas_subj_stats['conf']['old'])-2
# p_C_subj = scipy.stats.t.sf(np.float(t_r_C_subj),df_r_C_subj)
# In [365]: np.corrcoef(oldnew_areas_subj_stats['conf']['old'],oldnew_areas_subj_stats['conf']['new'])[0,1]
# Out[365]: 0.56615270048960631
# In [157]: t_r_C_subj
# Out[157]: 11.173995348598392
# In [158]: df_r_C_subj
# Out[158]: 169
# In [159]: p_C_subj
# Out[159]: 2.3312908120523647e-22

# # subj. based analyses

# evcat = 'all_recunrec'
# oldnew_areas_subj_stats=oldnew_areas_subj[evcat]
# SE_r_L_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_subj_stats['rt']['old'],oldnew_areas_subj_stats['rt']['new'])[0,1])/
#                       (len(oldnew_areas_subj_stats['rt']['old'])-2))
# t_r_L_subj = np.corrcoef(oldnew_areas_subj_stats['rt']['old'],
#                          oldnew_areas_subj_stats['rt']['new'])[0,1]/SE_r_L_subj
# df_r_L_subj = len(oldnew_areas_subj_stats['rt']['old'])-2
# p_L_subj = scipy.stats.t.sf(np.float(t_r_L_subj),df_r_L_subj)
# In [169]: t_r_L_subj
# Out[169]: 14.413285827237075
# In [170]: df_r_L_subj
# Out[170]: 169
# In [171]: p_L_subj
# Out[171]: 1.5637519710292001e-31


# evcat = 'all_targlure'
# oldnew_areas_subj_stats=oldnew_areas_subj[evcat]
# SE_r_L_subj = np.sqrt((1-np.corrcoef(
#     oldnew_areas_subj_stats['rt']['old'],oldnew_areas_subj_stats['rt']['new'])[0,1])/
#                       (len(oldnew_areas_subj_stats['rt']['old'])-2))
# t_r_L_subj = np.corrcoef(oldnew_areas_subj_stats['rt']['old'],
#                          oldnew_areas_subj_stats['rt']['new'])[0,1]/SE_r_L_subj
# df_r_L_subj = len(oldnew_areas_subj_stats['rt']['old'])-2
# p_L_subj = scipy.stats.t.sf(np.float(t_r_L_subj),df_r_L_subj)
# In [366]: np.corrcoef(oldnew_areas_subj_stats['rt']['old'],oldnew_areas_subj_stats['rt']['new'])[0,1]
# Out[366]: 0.67831704257591974
# In [173]: t_r_L_subj
# Out[173]: 15.547553311241913
# In [174]: df_r_L_subj
# Out[174]: 169
# In [175]: p_L_subj
# Out[175]: 1.0254381773702737e-34



# recunrec_dat = open('./data/recunrecANOVA_sess.tsv','w')
# recunrec_dat.write('ID\trecunrec\toldnew\tdv\tAUC\n')
# subjects = resprates_dict_agg['recalled']['indiv_resprates']['conf']['subject']
# for sess in range(1,len(areas_evcats['recalled']['all']['old']['conf'])):
#     subj = subjects[sess]
#     recunrec_dat.write(str(subj)+'\trecalled\told\tconf\t'+str(
#         areas_evcats['recalled']['all']['old']['conf'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\tunrecalled\told\tconf\t'+str(
#         areas_evcats['unrecalled']['all']['old']['conf'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\trecalled\tnew\tconf\t'+str(
#         areas_evcats['recalled']['all']['new']['conf'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\tunrecalled\tnew\tconf\t'+str(
#         areas_evcats['unrecalled']['all']['new']['conf'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\trecalled\told\trt\t'+str(
#         areas_evcats['recalled']['all']['old']['rt'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\tunrecalled\told\trt\t'+str(
#         areas_evcats['unrecalled']['all']['old']['rt'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\trecalled\tnew\trt\t'+str(
#         areas_evcats['recalled']['all']['new']['rt'][sess-1])+'\n')
#     recunrec_dat.write(str(subj)+'\tunrecalled\tnew\trt\t'+str(
#         areas_evcats['unrecalled']['all']['new']['rt'][sess-1])+'\n')
# recunrec_dat.close()

